import { Entity } from '@lastolivegames/becsy';
export declare enum Pen {
    SELECT = "select",
    HAND = "hand",
    DRAW_RECT = "draw-rect"
}
export declare class Canvas {
    /**
     * The canvas element. Pass in HTMLCanvasElement in the browser environment, OffscreenCanvas in the WebWorker environment,
     * and node-canvas in the Node.js environment.
     */
    element: HTMLCanvasElement | OffscreenCanvas;
    /**
     * The width of the canvas.
     */
    width: number;
    /**
     * The height of the canvas.
     */
    height: number;
    /**
     * Set the renderer, optional values are webgl and webgpu, default value is webgl.
     */
    renderer: 'webgl' | 'webgpu';
    /**
     * Set the WebGPU shader compiler path.
     */
    shaderCompilerPath: string;
    /**
     * Returns the ratio of the resolution in physical pixels to the resolution
     * in CSS pixels for the current display device.
     * @see https://developer.mozilla.org/en-US/docs/Web/API/Window/devicePixelRatio
     */
    devicePixelRatio: number;
    /**
     * Default to `Pen.HAND`.
     */
    pen: Pen;
    /**
     * TODO: multiple {@link Camera}s
     * @example
     *
     * camera1.write(Camera).canvas = canvas;
     * camera2.write(Camera).canvas = canvas;
     */
    cameras: Entity[];
    inputPoints: Entity[];
    screenshots: Entity[];
    vectorScreenshotRequests: Entity[];
    rasterScreenshotRequests: Entity[];
    constructor(canvas?: Partial<Canvas>);
}
