var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { field, Type } from '@lastolivegames/becsy';
import { AABB } from '../math';
export var TesselationMethod;
(function (TesselationMethod) {
    TesselationMethod["EARCUT"] = "earcut";
    TesselationMethod["LIBTESS"] = "libtess";
})(TesselationMethod || (TesselationMethod = {}));
/**
 * It is the generic element to define a shape. All the basic shapes can be created with a path element.
 * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Element/path
 */
export class Path {
    static getGeometryBounds(path, computed) {
        const { d } = path;
        const { points } = computed;
        if (!d) {
            return new AABB(0, 0, 0, 0);
        }
        const flattedPoints = points.flat();
        // FIXME: account for strokeLinejoin & strokeLinecap
        const minX = Math.min(...flattedPoints.map((point) => point[0]));
        const maxX = Math.max(...flattedPoints.map((point) => point[0]));
        const minY = Math.min(...flattedPoints.map((point) => point[1]));
        const maxY = Math.max(...flattedPoints.map((point) => point[1]));
        return new AABB(minX, minY, maxX, maxY);
    }
    static getRenderBounds(path, computed, stroke) {
        const { width = 0, linecap = 'butt' } = stroke !== null && stroke !== void 0 ? stroke : {};
        let style_expansion = 0.5;
        if (linecap === 'square') {
            style_expansion = Math.SQRT1_2;
        }
        // const stroke_is_rectilinear = true;
        // if (
        //   strokeLinejoin === 'miter' &&
        //   style_expansion < Math.SQRT2 * strokeMiterlimit &&
        //   !stroke_is_rectilinear
        // ) {
        //   style_expansion = Math.SQRT2 * strokeMiterlimit;
        // }
        style_expansion *= width;
        const { minX, minY, maxX, maxY } = Path.getGeometryBounds(path, computed);
        return new AABB(minX - style_expansion, minY - style_expansion, maxX + style_expansion, maxY + style_expansion);
    }
    constructor(props) {
        this.d = props === null || props === void 0 ? void 0 : props.d;
        this.fillRule = props === null || props === void 0 ? void 0 : props.fillRule;
        this.tessellationMethod = props === null || props === void 0 ? void 0 : props.tessellationMethod;
    }
}
__decorate([
    field({ type: Type.dynamicString(1000) })
], Path.prototype, "d", void 0);
__decorate([
    field({
        type: Type.staticString(['nonzero', 'evenodd']),
        default: 'nonzero',
    })
], Path.prototype, "fillRule", void 0);
__decorate([
    field({ type: Type.staticString(['earcut', 'libtess']), default: 'earcut' })
], Path.prototype, "tessellationMethod", void 0);
export class ComputedPoints {
}
__decorate([
    field({ type: Type.object })
], ComputedPoints.prototype, "points", void 0);
__decorate([
    field({ type: Type.object })
], ComputedPoints.prototype, "shiftedPoints", void 0);
//# sourceMappingURL=Path.js.map