var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { field, Type } from '@lastolivegames/becsy';
import { AABB } from '../math';
/**
 * Basic shape that creates straight lines connecting several points.
 * Typically a polyline is used to create open shapes as the last point doesn't have to be connected to the first point.
 * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Element/polyline
 */
export class Polyline {
    static getGeometryBounds(polyline) {
        const { points } = polyline;
        if (!points || points.length < 2) {
            return new AABB(0, 0, 0, 0);
        }
        // FIXME: account for strokeLinejoin & strokeLinecap
        const minX = Math.min(...points.map((point) => (isNaN(point[0]) ? Infinity : point[0])));
        const maxX = Math.max(...points.map((point) => (isNaN(point[0]) ? -Infinity : point[0])));
        const minY = Math.min(...points.map((point) => (isNaN(point[1]) ? Infinity : point[1])));
        const maxY = Math.max(...points.map((point) => (isNaN(point[1]) ? -Infinity : point[1])));
        return new AABB(minX, minY, maxX, maxY);
    }
    static getRenderBounds(polyline, stroke) {
        const { width, linecap } = stroke !== null && stroke !== void 0 ? stroke : {};
        let style_expansion = 0.5;
        if (linecap === 'square') {
            style_expansion = Math.SQRT1_2;
        }
        // const stroke_is_rectilinear = true;
        // if (
        //   strokeLinejoin === 'miter' &&
        //   style_expansion < Math.SQRT2 * strokeMiterlimit &&
        //   !stroke_is_rectilinear
        // ) {
        //   style_expansion = Math.SQRT2 * strokeMiterlimit;
        // }
        style_expansion *= width;
        const { minX, minY, maxX, maxY } = Polyline.getGeometryBounds(polyline);
        return new AABB(minX - style_expansion, minY - style_expansion, maxX + style_expansion, maxY + style_expansion);
    }
    constructor(props) {
        this.points = props === null || props === void 0 ? void 0 : props.points;
    }
}
__decorate([
    field({
        type: Type.object,
        default: [
            [0, 0],
            [0, 0],
        ],
    })
], Polyline.prototype, "points", void 0);
//# sourceMappingURL=Polyline.js.map