import { AABB } from '../math';
import { DropShadow, Stroke } from '../renderable';
/**
 * Defined by its position, width, and height. The rectangle may have its corners rounded.
 * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Element/rect
 */
export declare class Rect {
    static getGeometryBounds(rect: Rect): AABB;
    static getRenderBounds(rect: Rect, stroke?: Stroke, dropShadow?: DropShadow): AABB;
    /**
     * The x attribute defines an x-axis coordinate in the user coordinate system.
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/x
     */
    x: number;
    /**
     * The y attribute defines an x-axis coordinate in the user coordinate system.
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/y
     */
    y: number;
    /**
     * The width attribute defines the horizontal length of an element in the user coordinate system.
     * Negative values are allowed.
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/width
     *
     */
    width: number;
    /**
     * The height attribute defines the vertical length of an element in the user coordinate system.
     * Negative values are allowed.
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/height
     *
     */
    height: number;
    /**
     * For <rect>, cornerRadius used to round off the corners of the rectangle.
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/rx
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/ry
     */
    cornerRadius: number;
    constructor(props?: Partial<Rect>);
}
