var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
import { field, Type } from '@lastolivegames/becsy';
import { strokeOffset } from '../../utils';
import { yOffsetFromTextBaseline } from '../../systems/ComputeTextMetrics';
import { AABB } from '../math';
/**
 * Draws a graphics element consisting of text.
 * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Element/text
 */
export class Text {
    static getGeometryBounds(text, computed) {
        const { x, y, textAlign, textBaseline } = text;
        const { width, height, fontMetrics } = computed;
        const hwidth = width / 2;
        // default 'left'
        let lineXOffset = x;
        if (textAlign === 'center') {
            lineXOffset += -hwidth;
        }
        else if (textAlign === 'right' || textAlign === 'end') {
            lineXOffset += -hwidth * 2;
        }
        let lineYOffset = y;
        if (fontMetrics) {
            lineYOffset += yOffsetFromTextBaseline(textBaseline, fontMetrics);
        }
        return new AABB(lineXOffset, lineYOffset, lineXOffset + width, lineYOffset + height);
    }
    static getRenderBounds(text, computed, stroke, dropShadow) {
        const { offsetX = 0, offsetY = 0, blurRadius = 0 } = dropShadow !== null && dropShadow !== void 0 ? dropShadow : {};
        const offset = strokeOffset(stroke);
        const { minX, minY, maxX, maxY } = Text.getGeometryBounds(text, computed);
        const renderBounds = new AABB(minX - offset, minY - offset, maxX + offset, maxY + offset);
        renderBounds.addBounds(new AABB(minX + offsetX - blurRadius, minY + offsetY - blurRadius, maxX + offsetX + blurRadius, maxY + offsetY + blurRadius));
        return renderBounds;
    }
    constructor(props) {
        Object.assign(this, props);
    }
}
__decorate([
    field({ type: Type.float32, default: 0 })
], Text.prototype, "x", void 0);
__decorate([
    field({ type: Type.float32, default: 0 })
], Text.prototype, "y", void 0);
__decorate([
    field({ type: Type.dynamicString(1000), default: '' })
], Text.prototype, "content", void 0);
__decorate([
    field({ type: Type.dynamicString(100), default: 'sans-serif' })
], Text.prototype, "fontFamily", void 0);
__decorate([
    field({ type: Type.object, default: 12 })
], Text.prototype, "fontSize", void 0);
__decorate([
    field({ type: Type.object, default: 'normal' })
], Text.prototype, "fontWeight", void 0);
__decorate([
    field({ type: Type.dynamicString(100), default: 'normal' })
], Text.prototype, "fontStyle", void 0);
__decorate([
    field({ type: Type.dynamicString(100), default: 'normal' })
], Text.prototype, "fontVariant", void 0);
__decorate([
    field({ type: Type.float32, default: 0 })
], Text.prototype, "letterSpacing", void 0);
__decorate([
    field({
        type: Type.staticString(['normal', 'pre', 'pre-line']),
        default: 'normal',
    })
], Text.prototype, "whiteSpace", void 0);
__decorate([
    field({
        type: Type.staticString(['start', 'end', 'left', 'right', 'center']),
        default: 'start',
    })
], Text.prototype, "textAlign", void 0);
__decorate([
    field({
        type: Type.staticString([
            'top',
            'hanging',
            'middle',
            'alphabetic',
            'ideographic',
            'bottom',
        ]),
        default: 'alphabetic',
    })
], Text.prototype, "textBaseline", void 0);
__decorate([
    field({ type: Type.float32, default: 0 })
], Text.prototype, "lineHeight", void 0);
__decorate([
    field({ type: Type.float32, default: 0 })
], Text.prototype, "leading", void 0);
__decorate([
    field({ type: Type.object })
], Text.prototype, "bitmapFont", void 0);
__decorate([
    field({ type: Type.boolean, default: true })
], Text.prototype, "bitmapFontKerning", void 0);
__decorate([
    field({ type: Type.boolean, default: true })
], Text.prototype, "esdt", void 0);
__decorate([
    field({ type: Type.boolean, default: false })
], Text.prototype, "physical", void 0);
__decorate([
    field({ type: Type.boolean, default: false })
], Text.prototype, "wordWrap", void 0);
__decorate([
    field({ type: Type.float32, default: 0 })
], Text.prototype, "wordWrapWidth", void 0);
__decorate([
    field({ type: Type.staticString(['ellipsis', 'clip']) })
], Text.prototype, "textOverflow", void 0);
__decorate([
    field({ type: Type.int32, default: Infinity })
], Text.prototype, "maxLines", void 0);
export class ComputedTextMetrics {
}
__decorate([
    field({ type: Type.object })
], ComputedTextMetrics.prototype, "bidiChars", void 0);
__decorate([
    field({ type: Type.dynamicString(100), default: '' })
], ComputedTextMetrics.prototype, "font", void 0);
__decorate([
    field({ type: Type.float32, default: 0 })
], ComputedTextMetrics.prototype, "width", void 0);
__decorate([
    field({ type: Type.float32, default: 0 })
], ComputedTextMetrics.prototype, "height", void 0);
__decorate([
    field({ type: Type.object })
], ComputedTextMetrics.prototype, "lines", void 0);
__decorate([
    field({ type: Type.float32, default: [] })
], ComputedTextMetrics.prototype, "lineWidths", void 0);
__decorate([
    field({ type: Type.float32, default: 0 })
], ComputedTextMetrics.prototype, "lineHeight", void 0);
__decorate([
    field({ type: Type.float32, default: 0 })
], ComputedTextMetrics.prototype, "maxLineWidth", void 0);
__decorate([
    field({ type: Type.object })
], ComputedTextMetrics.prototype, "fontMetrics", void 0);
__decorate([
    field({ type: Type.object })
], ComputedTextMetrics.prototype, "lineMetrics", void 0);
//# sourceMappingURL=Text.js.map