import { System } from '@lastolivegames/becsy';
import { Circle, ComputedBounds, ComputedPoints, ComputedTextMetrics, DropShadow, Ellipse, GlobalTransform, Path, Polyline, Rect, Renderable, Stroke, Text, } from '../components';
export class ComputeBounds extends System {
    constructor() {
        super();
        this.renderables = this.query((q) => q.addedOrChanged
            .with(Renderable, GlobalTransform)
            .withAny(Circle, Ellipse, Rect, Polyline, Path, Text).trackWrites);
        this.query((q) => q.current.with(ComputedBounds).write);
        this.query((q) => q.using(Circle, Ellipse, Rect, Polyline, Path, ComputedPoints, Text, ComputedTextMetrics, Stroke, DropShadow).read);
    }
    execute() {
        this.renderables.addedOrChanged.forEach((entity) => {
            if (!entity.has(ComputedBounds)) {
                entity.add(ComputedBounds);
            }
            const stroke = entity.has(Stroke) ? entity.read(Stroke) : undefined;
            const dropShadow = entity.has(DropShadow)
                ? entity.read(DropShadow)
                : undefined;
            let geometryBounds;
            let renderBounds;
            if (entity.has(Circle)) {
                geometryBounds = Circle.getGeometryBounds(entity.read(Circle));
                renderBounds = Circle.getRenderBounds(entity.read(Circle), stroke);
            }
            else if (entity.has(Ellipse)) {
                geometryBounds = Ellipse.getGeometryBounds(entity.read(Ellipse));
                renderBounds = Ellipse.getRenderBounds(entity.read(Ellipse), stroke);
            }
            else if (entity.has(Rect)) {
                geometryBounds = Rect.getGeometryBounds(entity.read(Rect));
                renderBounds = Rect.getRenderBounds(entity.read(Rect), stroke, dropShadow);
            }
            else if (entity.has(Polyline)) {
                geometryBounds = Polyline.getGeometryBounds(Object.assign(Object.assign({}, entity.read(Polyline)), { points: entity.read(ComputedPoints).shiftedPoints }));
                renderBounds = Polyline.getRenderBounds(entity.read(Polyline), stroke);
            }
            else if (entity.has(Path)) {
                geometryBounds = Path.getGeometryBounds(entity.read(Path), entity.read(ComputedPoints));
                renderBounds = Path.getRenderBounds(entity.read(Path), entity.read(ComputedPoints), stroke);
            }
            else if (entity.has(Text)) {
                geometryBounds = Text.getGeometryBounds(entity.read(Text), entity.read(ComputedTextMetrics));
                renderBounds = Text.getRenderBounds(entity.read(Text), entity.read(ComputedTextMetrics), stroke, dropShadow);
            }
            if (geometryBounds) {
                entity.write(ComputedBounds).geometryBounds = geometryBounds;
            }
            if (renderBounds) {
                entity.write(ComputedBounds).renderBounds = renderBounds;
            }
        });
    }
}
//# sourceMappingURL=ComputeBounds.js.map