var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _EventWriter_onDestroyCallbacks;
import { co, System } from '@lastolivegames/becsy';
import { Canvas, Input } from '../components';
import { getGlobalThis } from '../utils';
import { Cursor } from '..';
/**
 * This system will bind event listeners to the canvas.
 * It will also handle the pointer events and keyboard events.
 */
export class EventWriter extends System {
    *setInputTrigger(entity, triggerKey) {
        const input = entity.write(Input);
        if (!(triggerKey in input))
            return;
        Object.assign(input, { [triggerKey]: true });
        yield;
        {
            const input = entity.hold().write(Input);
            Object.assign(input, { [triggerKey]: false });
        }
        yield;
    }
    constructor() {
        super();
        this.canvases = this.query((q) => q.added.and.removed.with(Canvas));
        this.pointerIds = new Map();
        _EventWriter_onDestroyCallbacks.set(this, new WeakMap());
        this.query((q) => q.using(Input, Cursor).write);
    }
    execute() {
        this.canvases.added.forEach((entity) => {
            if (!entity.has(Input)) {
                entity.add(Input);
            }
            if (!entity.has(Cursor)) {
                entity.add(Cursor);
            }
            this.bindEventListeners(entity);
        });
        this.canvases.removed.forEach((entity) => {
            var _a;
            this.accessRecentlyDeletedData();
            const canvas = entity.read(Canvas);
            const { element } = canvas;
            (_a = __classPrivateFieldGet(this, _EventWriter_onDestroyCallbacks, "f")
                .get(element)) === null || _a === void 0 ? void 0 : _a.forEach((callback) => callback());
        });
    }
    bindEventListeners(entity) {
        const canvas = entity.read(Canvas).element;
        const globalThis = getGlobalThis();
        const supportsPointerEvents = !!globalThis.PointerEvent;
        const supportsTouchEvents = 'ontouchstart' in globalThis;
        const input = entity.hold();
        this.pointerIds.set(entity.__id, new Set());
        const pointerIds = this.pointerIds.get(entity.__id);
        const onPointerMove = (e) => {
            // @see https://stackoverflow.com/questions/49500339/cant-prevent-touchmove-from-scrolling-window-on-ios
            // ev.preventDefault();
            if (pointerIds.size > 1 || !pointerIds.has(e.pointerId))
                return;
            const pointerWorld = this.client2Viewport({
                x: e.clientX,
                y: e.clientY,
            }, canvas);
            input.write(Input).pointerWorld = [pointerWorld.x, pointerWorld.y];
        };
        const onPointerUp = (e) => {
            // input.write(Input).pointerUpTrigger = true;
            this.setInputTrigger(input, 'pointerUpTrigger');
            pointerIds.delete(e.pointerId);
        };
        const onPointerDown = (e) => {
            const mouseButtons = [0, 1, 2];
            if (e.pointerType === 'mouse' && !mouseButtons.includes(e.button))
                return;
            pointerIds.add(e.pointerId);
            if (pointerIds.size > 1) {
                return;
            }
            // input.write(Input).pointerDownTrigger = true;
            this.setInputTrigger(input, 'pointerDownTrigger');
            if (pointerIds.size === 1) {
                const pointerWorld = this.client2Viewport({
                    x: e.clientX,
                    y: e.clientY,
                }, canvas);
                input.write(Input).pointerWorld = [pointerWorld.x, pointerWorld.y];
            }
        };
        const onPointerCancel = (e) => {
            pointerIds.delete(e.pointerId);
        };
        const onPointerWheel = (e) => {
            e.preventDefault();
            input.write(Input).wheelTrigger = true;
            input.write(Input).deltaX = e.deltaX;
            input.write(Input).deltaY = e.deltaY;
            if (e.ctrlKey) {
                input.write(Input).ctrlKey = true;
            }
            if (e.shiftKey) {
                input.write(Input).shiftKey = true;
            }
            if (e.metaKey) {
                input.write(Input).metaKey = true;
            }
            const pointerWorld = this.client2Viewport({
                x: e.clientX,
                y: e.clientY,
            }, canvas);
            input.write(Input).pointerWorld = [pointerWorld.x, pointerWorld.y];
        };
        const onKeyDown = (e) => {
            if (e.key === 'Control') {
                input.write(Input).ctrlKey = true;
            }
            if (e.key === 'Shift') {
                input.write(Input).shiftKey = true;
            }
            if (e.key === 'Meta') {
                input.write(Input).metaKey = true;
            }
        };
        const onKeyUp = (e) => {
            if (e.key === 'Control') {
                input.write(Input).ctrlKey = false;
            }
            if (e.key === 'Shift') {
                input.write(Input).shiftKey = false;
            }
            if (e.key === 'Meta') {
                input.write(Input).metaKey = false;
            }
        };
        const addPointerEventListener = ($el) => {
            $el.addEventListener('pointermove', onPointerMove, true);
            $el.addEventListener('pointerdown', onPointerDown, true);
            $el.addEventListener('pointerup', onPointerUp, true);
            $el.addEventListener('pointercancel', onPointerCancel, true);
        };
        const addTouchEventListener = ($el) => {
            $el.addEventListener('touchstart', onPointerDown, true);
            $el.addEventListener('touchend', onPointerUp, true);
            $el.addEventListener('touchmove', onPointerMove, true);
            $el.addEventListener('touchcancel', onPointerCancel, true);
        };
        const addMouseEventListener = ($el) => {
            $el.addEventListener('mousemove', onPointerMove, true);
            $el.addEventListener('mousedown', onPointerDown, true);
            $el.addEventListener('mouseup', onPointerUp, true);
        };
        const removePointerEventListener = ($el) => {
            $el.removeEventListener('pointermove', onPointerMove, true);
            $el.removeEventListener('pointerdown', onPointerDown, true);
            $el.removeEventListener('pointerup', onPointerUp, true);
            $el.removeEventListener('pointercancel', onPointerCancel, true);
        };
        const removeTouchEventListener = ($el) => {
            $el.removeEventListener('touchstart', onPointerDown, true);
            $el.removeEventListener('touchend', onPointerUp, true);
            $el.removeEventListener('touchmove', onPointerMove, true);
            $el.removeEventListener('touchcancel', onPointerCancel, true);
        };
        const removeMouseEventListener = ($el) => {
            $el.removeEventListener('mousemove', onPointerMove, true);
            $el.removeEventListener('mousedown', onPointerDown, true);
            $el.removeEventListener('mouseup', onPointerUp, true);
        };
        if ('addEventListener' in globalThis) {
            if (supportsPointerEvents) {
                addPointerEventListener(canvas);
            }
            else {
                addMouseEventListener(canvas);
                if (supportsTouchEvents) {
                    addTouchEventListener(canvas);
                }
            }
            // use passive event listeners
            // @see https://zhuanlan.zhihu.com/p/24555031
            canvas.addEventListener('wheel', onPointerWheel, {
                // passive: true,
                capture: true,
            });
            globalThis.addEventListener('keydown', onKeyDown, true);
            globalThis.addEventListener('keyup', onKeyUp, true);
            __classPrivateFieldGet(this, _EventWriter_onDestroyCallbacks, "f").set(canvas, [
                () => {
                    if (supportsPointerEvents) {
                        removePointerEventListener(canvas);
                    }
                    else {
                        removeMouseEventListener(canvas);
                        if (supportsTouchEvents) {
                            removeTouchEventListener(canvas);
                        }
                    }
                    canvas.removeEventListener('wheel', onPointerWheel, true);
                    globalThis.removeEventListener('keydown', onKeyDown, true);
                    globalThis.removeEventListener('keyup', onKeyUp, true);
                },
            ]);
        }
    }
    /**
     * Should account for CSS Transform applied on container.
     * @see https://github.com/antvis/G/issues/1161
     * @see https://github.com/antvis/G/issues/1677
     * @see https://developer.mozilla.org/zh-CN/docs/Web/API/MouseEvent/offsetX
     */
    client2Viewport({ x, y }, $el) {
        const { scaleX, scaleY, bbox } = this.getScale($el);
        return {
            x: (x - ((bbox === null || bbox === void 0 ? void 0 : bbox.left) || 0)) / scaleX,
            y: (y - ((bbox === null || bbox === void 0 ? void 0 : bbox.top) || 0)) / scaleY,
        };
    }
    viewport2Client({ x, y }, $el) {
        const { scaleX, scaleY, bbox } = this.getScale($el);
        return {
            x: (x + ((bbox === null || bbox === void 0 ? void 0 : bbox.left) || 0)) * scaleX,
            y: (y + ((bbox === null || bbox === void 0 ? void 0 : bbox.top) || 0)) * scaleY,
        };
    }
    getScale($el) {
        const bbox = $el.getBoundingClientRect();
        let scaleX = 1;
        let scaleY = 1;
        if ($el && bbox) {
            const { offsetWidth, offsetHeight } = $el;
            if (offsetWidth && offsetHeight) {
                scaleX = bbox.width / offsetWidth;
                scaleY = bbox.height / offsetHeight;
            }
        }
        return {
            scaleX,
            scaleY,
            bbox,
        };
    }
}
_EventWriter_onDestroyCallbacks = new WeakMap();
__decorate([
    co
], EventWriter.prototype, "setInputTrigger", null);
//# sourceMappingURL=EventWriter.js.map