var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { load } from '@loaders.gl/core';
import { ImageLoader } from '@loaders.gl/images';
import { BitmapFont } from './BitmapFont';
import { bitmapFontTextParser } from './bitmap-font-text-parser';
import { bitmapFontXMLStringParser } from './bitmap-font-xml-string-parser';
import { bitmapFontJSONParser } from './bitmap-font-json-parser';
const validExtensions = ['.xml', '.fnt'];
export const loadBitmapFont = {
    test(url) {
        return validExtensions.includes(url.split('.').pop().toLowerCase());
    },
    parse(asset) {
        return __awaiter(this, void 0, void 0, function* () {
            const bitmapFontData = bitmapFontJSONParser.test(asset)
                ? bitmapFontJSONParser.parse(asset)
                : bitmapFontTextParser.test(asset)
                    ? bitmapFontTextParser.parse(asset)
                    : bitmapFontXMLStringParser.parse(asset);
            const { pages } = bitmapFontData;
            const textureUrls = [];
            // if we have a distance field - we can assume this is a signed distance field font
            // and we should use force linear filtering and no alpha premultiply
            const textureOptions = bitmapFontData.distanceField
                ? {
                    scaleMode: 'linear',
                    alphaMode: 'premultiply-alpha-on-upload',
                    autoGenerateMipmaps: false,
                    resolution: 1,
                }
                : {};
            for (let i = 0; i < pages.length; ++i) {
                const pageFile = pages[i].file;
                const imagePath = pageFile;
                // imagePath = copySearchParams(imagePath, src);
                textureUrls.push({
                    src: imagePath,
                    data: textureOptions,
                });
            }
            const images = yield Promise.all(textureUrls.map((_a) => __awaiter(this, [_a], void 0, function* ({ src }) { return (yield load(src, ImageLoader)); })));
            return new BitmapFont({
                data: bitmapFontData,
                images,
            });
        });
    },
};
//# sourceMappingURL=load-bitmap-font.js.map