// Jenkins One-at-a-Time hash from http://www.burtleburtle.net/bob/hash/doobs.html
export function hashCodeNumberUpdate(hash, v = 0) {
    hash += v;
    hash += hash << 10;
    hash += hash >>> 6;
    return hash >>> 0;
}
export function hashCodeNumberFinish(hash) {
    hash += hash << 3;
    hash ^= hash >>> 11;
    hash += hash << 15;
    return hash >>> 0;
}
// Pass this as a hash function to use a one-bucket HashMap (equivalent to linear search in an array),
// which can be efficient for small numbers of items.
export function nullHashFunc() {
    return 0;
}
class HashBucket {
    constructor() {
        this.keys = [];
        this.values = [];
    }
}
export class HashMap {
    constructor(keyEqualFunc, keyHashFunc) {
        this.keyEqualFunc = keyEqualFunc;
        this.keyHashFunc = keyHashFunc;
        this.buckets = new Map();
    }
    findBucketIndex(bucket, k) {
        for (let i = 0; i < bucket.keys.length; i++)
            if (this.keyEqualFunc(k, bucket.keys[i]))
                return i;
        return -1;
    }
    findBucket(k) {
        const bw = this.keyHashFunc(k);
        return this.buckets.get(bw);
    }
    get(k) {
        const bucket = this.findBucket(k);
        if (bucket === undefined)
            return null;
        const bi = this.findBucketIndex(bucket, k);
        if (bi < 0)
            return null;
        return bucket.values[bi];
    }
    add(k, v) {
        const bw = this.keyHashFunc(k);
        if (this.buckets.get(bw) === undefined)
            this.buckets.set(bw, new HashBucket());
        const bucket = this.buckets.get(bw);
        bucket.keys.push(k);
        bucket.values.push(v);
    }
    delete(k) {
        const bucket = this.findBucket(k);
        if (bucket === undefined)
            return;
        const bi = this.findBucketIndex(bucket, k);
        if (bi === -1)
            return;
        bucket.keys.splice(bi, 1);
        bucket.values.splice(bi, 1);
    }
    clear() {
        this.buckets.clear();
    }
    size() {
        let acc = 0;
        for (const bucket of this.buckets.values())
            acc += bucket.values.length;
        return acc;
    }
    *values() {
        for (const bucket of this.buckets.values())
            for (let j = bucket.values.length - 1; j >= 0; j--)
                yield bucket.values[j];
    }
}
//# sourceMappingURL=hashmap.js.map