import { World } from '@lastolivegames/becsy';
import { Plugin } from './plugins';
/**
 * @see https://bevy-cheatbook.github.io/programming/app-builder.html
 */
export declare class App {
    #private;
    /**
     * The main ECS [`World`] of the [`App`].
     * This stores and provides access to all the main data of the application.
     * The systems of the [`App`] will run using this [`World`].
     */
    world: World;
    /**
     * @example
     * new App()
     *   .addPlugin(P1)
     */
    addPlugin(plugin: Plugin | [Plugin, any]): this;
    /**
     * @example
     * new App()
     *   .addPlugins(P1, P2)
     */
    addPlugins(...plugins: (Plugin | [Plugin, any])[]): this;
    /**
     * Start the app and run all systems.
     */
    run(): Promise<this>;
    /**
     * Exit the app.
     * @see https://bevy-cheatbook.github.io/programming/app-builder.html#quitting-the-app
     */
    exit(): Promise<void>;
}
