import { AABB } from '../math';
import { Stroke } from '../renderable';
/**
 * Used to create ellipses based on a center coordinate, and both their x and y radius.
 * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Element/ellipse
 */
export declare class Ellipse {
    static getGeometryBounds(ellipse: Ellipse): AABB;
    static getRenderBounds(ellipse: Ellipse, stroke?: Stroke): AABB;
    /**
     * The cx attribute define the x-axis coordinate of a center point.
     *
     * Default value is `0`.
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/cx
     */
    cx: number;
    /**
     * The cy attribute define the y-axis coordinate of a center point.
     *
     * Default value is `0`.
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/cy
     */
    cy: number;
    /**
     * The r attribute defines the radius of a ellipse.
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/rx
     *
     */
    rx: number;
    /**
     * The r attribute defines the radius of a ellipse.
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/ry
     *
     */
    ry: number;
    constructor(props?: Partial<Ellipse>);
}
