import { Texture } from '@antv/g-device-api';
import { type Pattern } from '../../utils';
export declare class FillSolid {
    /**
     * It's a presentation attribute that defines the color used to paint the element.
     *
     * Default to `black`.
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/fill
     */
    value: string;
    constructor(value?: string);
}
export declare class FillGradient {
    value: string;
    constructor(value?: string);
}
/**
 * A pattern using the specified image and repetition.
 * @see https://developer.mozilla.org/en-US/docs/Web/API/CanvasRenderingContext2D/createPattern
 */
export declare class FillPattern {
    /**
     * An image to be used as the pattern's image.
     *
     */
    image: string | CanvasImageSource;
    /**
     * A string indicating how to repeat the pattern's image.
     */
    repetition: 'repeat' | 'repeat-x' | 'repeat-y' | 'no-repeat';
    /**
     * Uses a DOMMatrix object as the pattern's transformation matrix and invokes it on the pattern.
     * @see https://developer.mozilla.org/en-US/docs/Web/API/CanvasPattern/setTransform
     */
    transform: string;
    constructor(value?: Pattern);
}
export declare class FillTexture {
    value: Texture;
    constructor(value?: Texture);
}
export declare class FillImage {
    src: TexImageSource;
    /**
     * @see https://developer.mozilla.org/zh-CN/docs/Web/CSS/object-fit
     * @see https://tympanus.net/codrops/2025/03/11/replicating-css-object-fit-in-webgl/
     */
    objectFit: 'contain' | 'cover' | 'fill' | 'none' | 'scale-down';
    constructor(value?: Partial<FillImage>);
}
