export declare class Stroke {
    /**
     * It is a presentation attribute defining the color used to paint the outline of the shape.
     *
     * Default to `none`.
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke
     */
    color: string;
    /**
     * It is a presentation attribute defining the width of the stroke to be applied to the shape.
     *
     * Default value is `1`.
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-width
     */
    width: number;
    /**
     * This property allows to align a stroke along the outline of the current object.
     *
     * Default value is `center`.
     *
     * @see https://www.w3.org/TR/svg-strokes/#SpecifyingStrokeAlignment
     *
     * * `center`: This value indicates that the stroke for each subpath is positioned along the outline of the current stroke. The extends of the stroke increase to both sides of the outline accordingly dependent on the `stroke-width`.
     * * `inner`: This value indicates that the stroke area is defined by the outline of each subpath of the current object and the computed value of the `stroke-width` property as offset orthogonal from the outline into the fill area of each subpath. The `stroke-linejoin` property must be ignored.
     * * `outer`: This value indicates that the stroke area is defined by the outline of each subpath of the current object and the computed value of the `stroke-width` property as offset orthogonal from the outline away from the fill area of each subpath.
     */
    alignment: 'center' | 'inner' | 'outer';
    /**
     * The stroke-linecap attribute is a presentation attribute defining the shape to be used at the end of open subpaths when they are stroked.
     *
     * Default value is `butt`.
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-linecap
     */
    linecap: CanvasLineCap;
    /**
     * The stroke-linejoin attribute is a presentation attribute defining the shape to be used at the corners of paths when they are stroked.
     *
     * Default value is `miter`.
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-linejoin
     */
    linejoin: CanvasLineJoin;
    /**
     * The stroke-miterlimit attribute is a presentation attribute defining a limit on the ratio of the miter length to the stroke-width used to draw a miter join.
     * When the limit is exceeded, the join is converted from a miter to a bevel.
     *
     * Default value is `4`.
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-miterlimit
     */
    miterlimit: number;
    /**
     * The stroke-dasharray attribute is a presentation attribute defining the pattern of dashes and gaps used to paint the outline of the shape;
     *
     * Default value is `[]`.
     *
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-dasharray
     */
    dasharray: [number, number];
    /**
     * The stroke-dashoffset attribute is a presentation attribute defining an offset on the rendering of the associated dash array.
     *
     * Default value is `0`.
     *
     * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-dashoffset
     */
    dashoffset: number;
    constructor(props?: Partial<Stroke>);
}
