import { Vec2 } from '../math/Vec2';
/**
 * A 2D transform.
 * @see https://github.com/tim-blackbird/bevy_mod_transform2d
 *
 * @example
 * ```ts
 * // Get the transform
 * const transform = entity.read(Transform);
 *
 * // Set the transform
 * entity.write(Transform).translation.x = 100;
 * ```
 */
export declare class Transform {
    /**
     * Position of the entity. In 2d, the last value of the `Vec3` is used for z-ordering.
     */
    translation: Vec2;
    /**
     * Scale of the entity.
     */
    scale: Vec2;
    /**
     * Rotation of the entity.
     */
    rotation: number;
    constructor(props?: Partial<{
        translation: {
            x: number;
            y: number;
        };
        scale: {
            x: number;
            y: number;
        };
        rotation: number;
    }>);
}
