import { Buffer, Device, InputLayoutBufferDescriptor, Program, RenderPass, RenderPipeline, InputLayout, Bindings } from '@antv/g-device-api';
import { Entity } from '@lastolivegames/becsy';
import { RenderCache } from '../utils';
import { TexturePool } from '../resources';
export declare const ZINDEX_FACTOR = 100000;
export declare abstract class Drawcall {
    protected device: Device;
    protected renderCache: RenderCache;
    protected texturePool: TexturePool;
    protected instanced: boolean;
    protected index: number;
    uid: number;
    shapes: Entity[];
    /**
     * Create a new batch if the number of instances exceeds.
     */
    protected maxInstances: number;
    geometryDirty: boolean;
    materialDirty: boolean;
    destroyed: boolean;
    protected program: Program;
    protected pipeline: RenderPipeline;
    protected inputLayout: InputLayout;
    protected bindings: Bindings;
    protected indexBuffer: Buffer;
    protected indexBufferData: Uint32Array;
    protected vertexBuffers: Buffer[];
    protected vertexBufferDatas: Float32Array[];
    protected vertexBufferOffsets: number[];
    protected vertexBufferDescriptors: InputLayoutBufferDescriptor[];
    protected barycentricBuffer: Buffer;
    protected barycentricBufferDescriptor: InputLayoutBufferDescriptor;
    constructor(device: Device, renderCache: RenderCache, texturePool: TexturePool, instanced: boolean, index: number);
    abstract createGeometry(): void;
    abstract createMaterial(define: string, uniformBuffer: Buffer): void;
    abstract render(renderPass: RenderPass, uniformLegacyObject: Record<string, unknown>): void;
    destroy(): void;
    validate(shape: Entity): boolean;
    submit(renderPass: RenderPass, uniformBuffer: Buffer, uniformLegacyObject: Record<string, unknown>): void;
    add(shape: Entity): void;
    remove(shape: Entity): void;
    count(): number;
    protected get useWireframe(): boolean;
    protected get useFillImage(): boolean;
    protected createProgram(vert: string, frag: string, defines: string): void;
    protected generateWireframe(): void;
}
