"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _Mesh_uniformBuffer, _Mesh_texture;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Mesh = void 0;
const g_device_api_1 = require("@antv/g-device-api");
const earcut_1 = __importDefault(require("earcut"));
const Drawcall_1 = require("./Drawcall");
const mesh_1 = require("../shaders/mesh");
const utils_1 = require("../utils");
const components_1 = require("../components");
const strokeAlignmentMap = {
    center: 0,
    inner: 1,
    outer: 2,
};
class Mesh extends Drawcall_1.Drawcall {
    constructor() {
        super(...arguments);
        _Mesh_uniformBuffer.set(this, void 0);
        _Mesh_texture.set(this, void 0);
        this.points = [];
        this.instanced = false;
    }
    static useDash(shape) {
        const { dasharray = [] } = shape.has(components_1.Stroke)
            ? shape.read(components_1.Stroke)
            : { dasharray: [] };
        return dasharray[0] > 0 && dasharray[1] > 0;
    }
    validate(shape) {
        const result = super.validate(shape);
        if (!result) {
            return false;
        }
        if (this.shapes.length === 0) {
            return true;
        }
        if (this.shapes[0].read(components_1.Path).d !== shape.read(components_1.Path).d) {
            return false;
        }
        const isInstanceFillImage = this.shapes[0].has(components_1.FillImage);
        const isShapeFillImage = shape.has(components_1.FillImage);
        if (isInstanceFillImage !== isShapeFillImage) {
            return false;
        }
        if (isInstanceFillImage && isShapeFillImage) {
            return this.shapes[0].read(components_1.FillImage).src === shape.read(components_1.FillImage).src;
        }
        return true;
    }
    createGeometry() {
        const instance = this.shapes[0];
        let rawPoints;
        let tessellationMethod;
        if (instance.has(components_1.Path)) {
            rawPoints = instance.read(components_1.ComputedPoints).points;
            tessellationMethod = instance.read(components_1.Path).tessellationMethod;
        }
        else if (instance.has(components_1.Rough) &&
            instance.has(components_1.ComputedRough) &&
            instance.hasSomeOf(components_1.Circle, components_1.Ellipse, components_1.Rect, components_1.Path)) {
            rawPoints = instance.read(components_1.ComputedRough).fillPathPoints;
            tessellationMethod = components_1.TesselationMethod.EARCUT;
        }
        const points = rawPoints.flat(2);
        if (points.length === 0) {
            return;
        }
        if (tessellationMethod === components_1.TesselationMethod.EARCUT) {
            let holes = [];
            let contours = [];
            const indices = [];
            let indexOffset = 0;
            let firstClockWise = (0, utils_1.isClockWise)(rawPoints[0]);
            rawPoints.forEach((points) => {
                const isHole = (0, utils_1.isClockWise)(points) !== firstClockWise;
                if (isHole) {
                    holes.push(contours.length);
                }
                else {
                    firstClockWise = (0, utils_1.isClockWise)(points);
                    if (holes.length > 0) {
                        indices.push(...(0, earcut_1.default)(contours.flat(), holes).map((i) => i + indexOffset));
                        indexOffset += contours.length;
                        holes = [];
                        contours = [];
                    }
                }
                contours.push(...points);
            });
            if (contours.length) {
                indices.push(...(0, earcut_1.default)(contours.flat(), holes).map((i) => i + indexOffset));
            }
            this.indexBufferData = new Uint32Array(indices);
            this.points = points;
            // const err = deviation(vertices, holes, dimensions, indices);
        }
        else if (tessellationMethod === components_1.TesselationMethod.LIBTESS) {
            const newPoints = (0, utils_1.triangulate)(rawPoints, instance.read(components_1.Path).fillRule);
            this.indexBufferData = new Uint32Array(new Array(newPoints.length / 2).fill(undefined).map((_, i) => i));
            this.points = newPoints;
        }
        if (this.vertexBuffers[0]) {
            this.vertexBuffers[0].destroy();
        }
        this.vertexBufferDatas[0] = new Float32Array(this.points);
        this.vertexBuffers[0] = this.device.createBuffer({
            viewOrSize: this.vertexBufferDatas[0],
            usage: g_device_api_1.BufferUsage.VERTEX,
            hint: g_device_api_1.BufferFrequencyHint.STATIC,
        });
        if (this.indexBuffer) {
            this.indexBuffer.destroy();
        }
        this.indexBuffer = this.device.createBuffer({
            viewOrSize: this.indexBufferData,
            usage: g_device_api_1.BufferUsage.INDEX,
            hint: g_device_api_1.BufferFrequencyHint.STATIC,
        });
        this.vertexBufferDescriptors = [
            {
                arrayStride: 4 * 2,
                stepMode: g_device_api_1.VertexStepMode.VERTEX,
                attributes: [
                    {
                        shaderLocation: mesh_1.Location.POSITION, // a_Position
                        offset: 0,
                        format: g_device_api_1.Format.F32_RG,
                    },
                ],
            },
        ];
    }
    createMaterial(defines, uniformBuffer) {
        if (this.points.length === 0) {
            return;
        }
        this.createProgram(mesh_1.vert, mesh_1.frag, defines);
        if (!__classPrivateFieldGet(this, _Mesh_uniformBuffer, "f")) {
            __classPrivateFieldSet(this, _Mesh_uniformBuffer, this.device.createBuffer({
                viewOrSize: Float32Array.BYTES_PER_ELEMENT * (16 + 4 + 4 + 4 + 4 + 4),
                usage: g_device_api_1.BufferUsage.UNIFORM,
                hint: g_device_api_1.BufferFrequencyHint.DYNAMIC,
            }), "f");
        }
        this.pipeline = this.renderCache.createRenderPipeline({
            inputLayout: this.inputLayout,
            program: this.program,
            colorAttachmentFormats: [g_device_api_1.Format.U8_RGBA_RT],
            depthStencilAttachmentFormat: g_device_api_1.Format.D24_S8,
            topology: g_device_api_1.PrimitiveTopology.TRIANGLES,
            megaStateDescriptor: {
                attachmentsState: [
                    {
                        channelWriteMask: g_device_api_1.ChannelWriteMask.ALL,
                        rgbBlendState: {
                            blendMode: g_device_api_1.BlendMode.ADD,
                            blendSrcFactor: g_device_api_1.BlendFactor.SRC_ALPHA,
                            blendDstFactor: g_device_api_1.BlendFactor.ONE_MINUS_SRC_ALPHA,
                        },
                        alphaBlendState: {
                            blendMode: g_device_api_1.BlendMode.ADD,
                            blendSrcFactor: g_device_api_1.BlendFactor.ONE,
                            blendDstFactor: g_device_api_1.BlendFactor.ONE_MINUS_SRC_ALPHA,
                        },
                    },
                ],
                blendConstant: g_device_api_1.TransparentBlack,
                depthWrite: true,
                depthCompare: g_device_api_1.CompareFunction.GREATER,
                stencilWrite: false,
                stencilFront: {
                    compare: g_device_api_1.CompareFunction.ALWAYS,
                    passOp: g_device_api_1.StencilOp.KEEP,
                    failOp: g_device_api_1.StencilOp.KEEP,
                    depthFailOp: g_device_api_1.StencilOp.KEEP,
                },
                stencilBack: {
                    compare: g_device_api_1.CompareFunction.ALWAYS,
                    passOp: g_device_api_1.StencilOp.KEEP,
                    failOp: g_device_api_1.StencilOp.KEEP,
                    depthFailOp: g_device_api_1.StencilOp.KEEP,
                },
            },
        });
        this.device.setResourceName(this.pipeline, 'MeshPipeline');
        const bindings = {
            pipeline: this.pipeline,
            uniformBufferBindings: [
                {
                    buffer: uniformBuffer,
                },
                {
                    buffer: __classPrivateFieldGet(this, _Mesh_uniformBuffer, "f"),
                },
            ],
        };
        this.bindings = this.renderCache.createBindings(bindings);
    }
    render(renderPass, sceneUniformLegacyObject) {
        if (this.points.length === 0) {
            return;
        }
        // if (
        //   this.shapes.some((shape) => shape.renderDirtyFlag) ||
        //   this.geometryDirty
        // ) {
        const { matrix } = this.shapes[0].read(components_1.GlobalTransform);
        const [buffer, legacyObject] = this.generateBuffer(this.shapes[0]);
        const u_ModelMatrix = [
            matrix.m00,
            matrix.m01,
            matrix.m02,
            matrix.m10,
            matrix.m11,
            matrix.m12,
            matrix.m20,
            matrix.m21,
            matrix.m22,
        ];
        __classPrivateFieldGet(this, _Mesh_uniformBuffer, "f").setSubData(0, new Uint8Array(new Float32Array([
            ...(0, utils_1.paddingMat3)(components_1.Mat3.fromGLMat3(u_ModelMatrix)),
            ...buffer,
        ]).buffer));
        const uniformLegacyObject = Object.assign({ u_ModelMatrix }, legacyObject);
        if (this.useFillImage) {
            uniformLegacyObject.u_FillImage = __classPrivateFieldGet(this, _Mesh_texture, "f");
        }
        this.program.setUniformsLegacy(uniformLegacyObject);
        if (this.useWireframe) {
            this.generateWireframe();
        }
        // }
        this.program.setUniformsLegacy(sceneUniformLegacyObject);
        renderPass.setPipeline(this.pipeline);
        const vertexBuffers = this.vertexBuffers.map((buffer) => ({ buffer }));
        if (this.useWireframe) {
            vertexBuffers.push({ buffer: this.barycentricBuffer });
        }
        renderPass.setVertexInput(this.inputLayout, vertexBuffers, {
            buffer: this.indexBuffer,
        });
        renderPass.setBindings(this.bindings);
        renderPass.drawIndexed(this.indexBufferData.length, this.shapes.length);
    }
    destroy() {
        var _a, _b;
        super.destroy();
        if (this.program) {
            (_a = __classPrivateFieldGet(this, _Mesh_uniformBuffer, "f")) === null || _a === void 0 ? void 0 : _a.destroy();
            (_b = __classPrivateFieldGet(this, _Mesh_texture, "f")) === null || _b === void 0 ? void 0 : _b.destroy();
        }
    }
    generateBuffer(shape) {
        const sizeAttenuation = 0;
        const globalRenderOrder = shape.has(components_1.GlobalRenderOrder)
            ? shape.read(components_1.GlobalRenderOrder).value
            : 0;
        const { value: fill } = shape.has(components_1.FillSolid)
            ? shape.read(components_1.FillSolid)
            : { value: null };
        const { r: fr, g: fg, b: fb, opacity: fo } = (0, utils_1.parseColor)(fill);
        const { opacity, strokeOpacity, fillOpacity } = shape.has(components_1.Opacity)
            ? shape.read(components_1.Opacity)
            : { opacity: 1, strokeOpacity: 1, fillOpacity: 1 };
        const { color: strokeColor, width, alignment, } = shape.has(components_1.Stroke)
            ? shape.read(components_1.Stroke)
            : { color: null, width: 0, alignment: 'center' };
        const { r: sr, g: sg, b: sb, opacity: so } = (0, utils_1.parseColor)(strokeColor);
        const u_FillColor = [fr / 255, fg / 255, fb / 255, fo];
        const u_StrokeColor = [sr / 255, sg / 255, sb / 255, so];
        const u_ZIndexStrokeWidth = [
            globalRenderOrder / Drawcall_1.ZINDEX_FACTOR,
            Mesh.useDash(shape) ? 0 : width,
            0,
            strokeAlignmentMap[alignment],
        ];
        const u_Opacity = [
            opacity,
            fillOpacity,
            strokeOpacity,
            sizeAttenuation ? 1 : 0,
        ];
        return [
            [...u_FillColor, ...u_StrokeColor, ...u_ZIndexStrokeWidth, ...u_Opacity],
            {
                u_FillColor,
                u_StrokeColor,
                u_ZIndexStrokeWidth,
                u_Opacity,
            },
        ];
    }
}
exports.Mesh = Mesh;
_Mesh_uniformBuffer = new WeakMap(), _Mesh_texture = new WeakMap();
//# sourceMappingURL=Mesh.js.map