"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var _SDF_uniformBuffer, _SDF_texture;
Object.defineProperty(exports, "__esModule", { value: true });
exports.SDF = void 0;
const g_device_api_1 = require("@antv/g-device-api");
const Drawcall_1 = require("./Drawcall");
const sdf_1 = require("../shaders/sdf");
const utils_1 = require("../utils");
const components_1 = require("../components");
const strokeAlignmentMap = {
    center: 0,
    inner: 1,
    outer: 2,
};
class SDF extends Drawcall_1.Drawcall {
    constructor() {
        // protected maxInstances: number = 1000;
        super(...arguments);
        _SDF_uniformBuffer.set(this, void 0);
        _SDF_texture.set(this, void 0);
    }
    static useDash(shape) {
        const { dasharray } = shape.has(components_1.Stroke)
            ? shape.read(components_1.Stroke)
            : { dasharray: [0, 0] };
        return dasharray[0] > 0 && dasharray[1] > 0;
    }
    validate(shape) {
        const result = super.validate(shape);
        if (!result) {
            return false;
        }
        if (this.shapes.length === 0) {
            return true;
        }
        const isInstanceFillImage = this.shapes[0].has(components_1.FillImage);
        const isShapeFillImage = shape.has(components_1.FillImage);
        if (isInstanceFillImage !== isShapeFillImage) {
            return false;
        }
        const isInstanceFillGradient = this.shapes[0].has(components_1.FillGradient);
        const isShapeFillGradient = shape.has(components_1.FillGradient);
        if (isInstanceFillGradient !== isShapeFillGradient) {
            return false;
        }
        if (isInstanceFillImage && isShapeFillImage) {
            return this.shapes[0].read(components_1.FillImage).src === shape.read(components_1.FillImage).src;
        }
        if (isInstanceFillGradient && isShapeFillGradient) {
            return this.shapes[0].read(components_1.FillGradient) === shape.read(components_1.FillGradient);
        }
        if (SDF.useDash(shape) !== SDF.useDash(this.shapes[0])) {
            return false;
        }
        return true;
    }
    createGeometry() {
        this.indexBufferData = new Uint32Array([0, 1, 2, 0, 2, 3]);
        if (this.indexBuffer) {
            this.indexBuffer.destroy();
        }
        this.indexBuffer = this.device.createBuffer({
            viewOrSize: this.indexBufferData,
            usage: g_device_api_1.BufferUsage.INDEX,
            hint: g_device_api_1.BufferFrequencyHint.STATIC,
        });
        this.vertexBufferDatas[0] = new Float32Array([-1, -1, 1, -1, 1, 1, -1, 1]);
        if (this.vertexBuffers[0]) {
            this.vertexBuffers[0].destroy();
        }
        this.vertexBuffers[0] = this.device.createBuffer({
            viewOrSize: this.vertexBufferDatas[0],
            usage: g_device_api_1.BufferUsage.VERTEX,
            hint: g_device_api_1.BufferFrequencyHint.STATIC,
        });
        if (this.instanced) {
            if (this.vertexBuffers[1]) {
                this.vertexBuffers[1].destroy();
                this.vertexBuffers[2].destroy();
            }
            this.vertexBufferDatas[1] = new Float32Array(new Array(32 * this.shapes.length).fill(0));
            this.vertexBuffers[1] = this.device.createBuffer({
                viewOrSize: this.vertexBufferDatas[1],
                usage: g_device_api_1.BufferUsage.VERTEX,
                hint: g_device_api_1.BufferFrequencyHint.DYNAMIC,
            });
            this.vertexBufferDatas[2] = new Float32Array(new Array(6 * this.shapes.length).fill(0));
            this.vertexBuffers[2] = this.device.createBuffer({
                viewOrSize: this.vertexBufferDatas[2],
                usage: g_device_api_1.BufferUsage.VERTEX,
                hint: g_device_api_1.BufferFrequencyHint.DYNAMIC,
            });
        }
        this.vertexBufferDescriptors = [
            {
                arrayStride: 4 * 2,
                stepMode: g_device_api_1.VertexStepMode.VERTEX,
                attributes: [
                    {
                        shaderLocation: sdf_1.Location.FRAG_COORD, // a_FragCoord
                        offset: 0,
                        format: g_device_api_1.Format.F32_RG,
                    },
                ],
            },
        ];
        if (this.instanced) {
            this.vertexBufferDescriptors.push({
                arrayStride: 4 * 32,
                stepMode: g_device_api_1.VertexStepMode.INSTANCE,
                attributes: [
                    {
                        shaderLocation: sdf_1.Location.POSITION, // a_Position
                        offset: 0,
                        format: g_device_api_1.Format.F32_RGBA,
                    },
                    {
                        shaderLocation: sdf_1.Location.SIZE, // a_Size
                        offset: 4 * 4,
                        format: g_device_api_1.Format.F32_RGBA,
                    },
                    {
                        shaderLocation: sdf_1.Location.FILL_COLOR, // a_FillColor
                        offset: 4 * 8,
                        format: g_device_api_1.Format.F32_RGBA,
                    },
                    {
                        shaderLocation: sdf_1.Location.STROKE_COLOR, // a_StrokeColor
                        offset: 4 * 12,
                        format: g_device_api_1.Format.F32_RGBA,
                    },
                    {
                        shaderLocation: sdf_1.Location.ZINDEX_STROKE_WIDTH, // a_ZIndexStrokeWidth
                        offset: 4 * 16,
                        format: g_device_api_1.Format.F32_RGBA,
                    },
                    {
                        shaderLocation: sdf_1.Location.OPACITY, // a_Opacity
                        offset: 4 * 20,
                        format: g_device_api_1.Format.F32_RGBA,
                    },
                    {
                        shaderLocation: sdf_1.Location.INNER_SHADOW_COLOR, // a_InnerShadowColor
                        offset: 4 * 24,
                        format: g_device_api_1.Format.F32_RGBA,
                    },
                    {
                        shaderLocation: sdf_1.Location.INNER_SHADOW, // a_InnerShadow
                        offset: 4 * 28,
                        format: g_device_api_1.Format.F32_RGBA,
                    },
                ],
            }, {
                arrayStride: 4 * 6,
                stepMode: g_device_api_1.VertexStepMode.INSTANCE,
                attributes: [
                    {
                        shaderLocation: sdf_1.Location.ABCD,
                        offset: 0,
                        format: g_device_api_1.Format.F32_RGBA,
                    },
                    {
                        shaderLocation: sdf_1.Location.TXTY,
                        offset: 4 * 4,
                        format: g_device_api_1.Format.F32_RG,
                    },
                ],
            });
        }
    }
    createMaterial(defines, uniformBuffer) {
        this.createProgram(sdf_1.vert, sdf_1.frag, defines);
        if (!this.instanced && !__classPrivateFieldGet(this, _SDF_uniformBuffer, "f")) {
            __classPrivateFieldSet(this, _SDF_uniformBuffer, this.device.createBuffer({
                viewOrSize: Float32Array.BYTES_PER_ELEMENT * (16 + 4 * 8),
                usage: g_device_api_1.BufferUsage.UNIFORM,
                hint: g_device_api_1.BufferFrequencyHint.DYNAMIC,
            }), "f");
        }
        this.pipeline = this.renderCache.createRenderPipeline({
            inputLayout: this.inputLayout,
            program: this.program,
            colorAttachmentFormats: [g_device_api_1.Format.U8_RGBA_RT],
            depthStencilAttachmentFormat: g_device_api_1.Format.D24_S8,
            megaStateDescriptor: {
                attachmentsState: [
                    {
                        channelWriteMask: g_device_api_1.ChannelWriteMask.ALL,
                        rgbBlendState: {
                            blendMode: g_device_api_1.BlendMode.ADD,
                            blendSrcFactor: g_device_api_1.BlendFactor.SRC_ALPHA,
                            blendDstFactor: g_device_api_1.BlendFactor.ONE_MINUS_SRC_ALPHA,
                        },
                        alphaBlendState: {
                            blendMode: g_device_api_1.BlendMode.ADD,
                            blendSrcFactor: g_device_api_1.BlendFactor.ONE,
                            blendDstFactor: g_device_api_1.BlendFactor.ONE_MINUS_SRC_ALPHA,
                        },
                    },
                ],
                blendConstant: g_device_api_1.TransparentBlack,
                depthWrite: true,
                depthCompare: g_device_api_1.CompareFunction.GREATER,
                stencilWrite: false,
                stencilFront: {
                    compare: g_device_api_1.CompareFunction.ALWAYS,
                    passOp: g_device_api_1.StencilOp.KEEP,
                    failOp: g_device_api_1.StencilOp.KEEP,
                    depthFailOp: g_device_api_1.StencilOp.KEEP,
                },
                stencilBack: {
                    compare: g_device_api_1.CompareFunction.ALWAYS,
                    passOp: g_device_api_1.StencilOp.KEEP,
                    failOp: g_device_api_1.StencilOp.KEEP,
                    depthFailOp: g_device_api_1.StencilOp.KEEP,
                },
            },
        });
        this.device.setResourceName(this.pipeline, 'SDFPipeline');
        const bindings = {
            pipeline: this.pipeline,
            uniformBufferBindings: [
                {
                    buffer: uniformBuffer,
                },
            ],
        };
        if (!this.instanced) {
            bindings.uniformBufferBindings.push({
                buffer: __classPrivateFieldGet(this, _SDF_uniformBuffer, "f"),
            });
        }
        if (this.useFillImage) {
            if (this.bindings) {
                this.bindings.destroy();
            }
            const instance = this.shapes[0];
            if (instance.has(components_1.FillGradient) || instance.has(components_1.FillPattern)) {
                const { minX, minY, maxX, maxY } = instance.read(components_1.ComputedBounds).geometryBounds;
                const width = maxX - minX;
                const height = maxY - minY;
                const canvas = instance.has(components_1.FillPattern)
                    ? this.texturePool.getOrCreatePattern({
                        pattern: instance.read(components_1.FillPattern),
                        width,
                        height,
                    })
                    : this.texturePool.getOrCreateGradient({
                        gradients: (0, utils_1.parseGradient)(instance.read(components_1.FillGradient).value),
                        min: [minX, minY],
                        width,
                        height,
                    });
                const texture = this.device.createTexture({
                    format: g_device_api_1.Format.U8_RGBA_NORM,
                    width: 128,
                    height: 128,
                    usage: g_device_api_1.TextureUsage.SAMPLED,
                });
                texture.setImageData([canvas]);
                __classPrivateFieldSet(this, _SDF_texture, texture, "f");
            }
            else if (instance.has(components_1.FillTexture)) {
                __classPrivateFieldSet(this, _SDF_texture, instance.read(components_1.FillTexture).value, "f");
            }
            else if (instance.has(components_1.FillImage)) {
                const src = instance.read(components_1.FillImage).src;
                const texture = this.device.createTexture({
                    format: g_device_api_1.Format.U8_RGBA_NORM,
                    width: src.width,
                    height: src.height,
                    usage: g_device_api_1.TextureUsage.SAMPLED,
                });
                texture.setImageData([src]);
                __classPrivateFieldSet(this, _SDF_texture, texture, "f");
            }
            const sampler = this.renderCache.createSampler({
                addressModeU: g_device_api_1.AddressMode.CLAMP_TO_EDGE,
                addressModeV: g_device_api_1.AddressMode.CLAMP_TO_EDGE,
                minFilter: g_device_api_1.FilterMode.POINT,
                magFilter: g_device_api_1.FilterMode.BILINEAR,
                mipmapFilter: g_device_api_1.MipmapFilterMode.LINEAR,
                lodMinClamp: 0,
                lodMaxClamp: 0,
            });
            bindings.samplerBindings = [
                {
                    texture: __classPrivateFieldGet(this, _SDF_texture, "f"),
                    sampler,
                },
            ];
        }
        this.bindings = this.renderCache.createBindings(bindings);
    }
    render(renderPass, sceneUniformLegacyObject) {
        // if (
        //   this.shapes.some((shape) => shape.renderDirtyFlag) ||
        //   this.geometryDirty
        // ) {
        if (this.instanced) {
            const instancedData = [];
            this.shapes.forEach((shape, i, total) => {
                const [buffer] = this.generateBuffer(shape, i, total.length);
                instancedData.push(...buffer);
            });
            this.vertexBufferDatas[1] = new Float32Array(instancedData);
            this.vertexBuffers[1].setSubData(0, new Uint8Array(this.vertexBufferDatas[1].buffer));
            this.vertexBufferDatas[2] = new Float32Array(this.shapes
                .map((shape) => {
                const { matrix } = shape.read(components_1.GlobalTransform);
                return [
                    matrix.m00,
                    matrix.m01,
                    matrix.m02,
                    matrix.m10,
                    matrix.m11,
                    matrix.m12,
                ];
            })
                .flat());
            this.vertexBuffers[2].setSubData(0, new Uint8Array(this.vertexBufferDatas[2].buffer));
        }
        else {
            const { matrix } = this.shapes[0].read(components_1.GlobalTransform);
            const [buffer, legacyObject] = this.generateBuffer(this.shapes[0], 0, 1);
            const u_ModelMatrix = [
                matrix.m00,
                matrix.m01,
                matrix.m02,
                matrix.m10,
                matrix.m11,
                matrix.m12,
                matrix.m20,
                matrix.m21,
                matrix.m22,
            ];
            __classPrivateFieldGet(this, _SDF_uniformBuffer, "f").setSubData(0, new Uint8Array(new Float32Array([
                ...(0, utils_1.paddingMat3)(components_1.Mat3.fromGLMat3(u_ModelMatrix)),
                ...buffer,
            ]).buffer));
            const uniformLegacyObject = Object.assign({ u_ModelMatrix }, legacyObject);
            if (this.useFillImage) {
                uniformLegacyObject.u_FillImage = __classPrivateFieldGet(this, _SDF_texture, "f");
            }
            this.program.setUniformsLegacy(uniformLegacyObject);
        }
        if (this.useWireframe) {
            this.generateWireframe();
            if (this.instanced) {
                this.vertexBuffers[1].setSubData(0, new Uint8Array(this.vertexBufferDatas[1].buffer));
                this.vertexBuffers[2].setSubData(0, new Uint8Array(this.vertexBufferDatas[2].buffer));
            }
        }
        // }
        this.program.setUniformsLegacy(sceneUniformLegacyObject);
        renderPass.setPipeline(this.pipeline);
        const vertexBuffers = this.vertexBuffers.map((buffer) => ({ buffer }));
        if (this.useWireframe) {
            vertexBuffers.push({ buffer: this.barycentricBuffer });
        }
        renderPass.setVertexInput(this.inputLayout, vertexBuffers, {
            buffer: this.indexBuffer,
        });
        renderPass.setBindings(this.bindings);
        renderPass.drawIndexed(6, this.shapes.length);
    }
    destroy() {
        var _a, _b, _c;
        super.destroy();
        if (this.program) {
            (_a = __classPrivateFieldGet(this, _SDF_uniformBuffer, "f")) === null || _a === void 0 ? void 0 : _a.destroy();
            (_c = (_b = __classPrivateFieldGet(this, _SDF_texture, "f")) === null || _b === void 0 ? void 0 : _b.destroy) === null || _c === void 0 ? void 0 : _c.call(_b);
        }
    }
    generateBuffer(shape, index, total) {
        // TODO
        const sizeAttenuation = 0;
        const globalRenderOrder = shape.has(components_1.GlobalRenderOrder)
            ? shape.read(components_1.GlobalRenderOrder).value
            : 0;
        let position = [0, 0, 0, 0];
        let size = [0, 0, 0, 0];
        let type = 0;
        let cornerRadius = 0;
        const zIndex = (globalRenderOrder + (1 / total) * index) / Drawcall_1.ZINDEX_FACTOR;
        if (shape.has(components_1.Circle)) {
            const { cx, cy, r } = shape.read(components_1.Circle);
            position = [cx, cy, zIndex, 0];
            size = [r, r, 0, 0];
            type = 0;
        }
        else if (shape.has(components_1.Ellipse)) {
            const { cx, cy, rx, ry } = shape.read(components_1.Ellipse);
            position = [cx, cy, zIndex, 0];
            size = [rx, ry, 0, 0];
            type = 1;
        }
        else if (shape.has(components_1.Rect)) {
            const { x, y, width, height, cornerRadius: r } = shape.read(components_1.Rect);
            position = [x + width / 2, y + height / 2, zIndex, 0];
            size = [width / 2, height / 2, 0, 0];
            type = 2;
            cornerRadius = r;
        }
        const { value: fill } = shape.has(components_1.FillSolid)
            ? shape.read(components_1.FillSolid)
            : { value: null };
        const { r: fr, g: fg, b: fb, opacity: fo } = (0, utils_1.parseColor)(fill);
        const { opacity, strokeOpacity, fillOpacity } = shape.has(components_1.Opacity)
            ? shape.read(components_1.Opacity)
            : { opacity: 1, strokeOpacity: 1, fillOpacity: 1 };
        const { color: strokeColor, width, alignment, } = shape.has(components_1.Stroke)
            ? shape.read(components_1.Stroke)
            : { color: null, width: 0, alignment: 'center' };
        const { color: innerShadowColor, offsetX, offsetY, blurRadius, } = shape.has(components_1.InnerShadow)
            ? shape.read(components_1.InnerShadow)
            : {
                color: null,
                offsetX: 0,
                offsetY: 0,
                blurRadius: 0,
            };
        const { r: sr, g: sg, b: sb, opacity: so } = (0, utils_1.parseColor)(strokeColor);
        const { r: isr, g: isg, b: isb, opacity: iso, } = (0, utils_1.parseColor)(innerShadowColor);
        const u_Position = position;
        const u_Size = size;
        const u_FillColor = [fr / 255, fg / 255, fb / 255, fo];
        const u_StrokeColor = [sr / 255, sg / 255, sb / 255, so];
        const u_ZIndexStrokeWidth = [
            globalRenderOrder / Drawcall_1.ZINDEX_FACTOR,
            SDF.useDash(shape) ? 0 : width,
            cornerRadius,
            strokeAlignmentMap[alignment],
        ];
        const LEFT_SHIFT23 = 8388608.0;
        const compressed = (sizeAttenuation ? 1 : 0) * LEFT_SHIFT23 + type;
        const u_Opacity = [opacity, fillOpacity, strokeOpacity, compressed];
        const u_InnerShadowColor = [isr / 255, isg / 255, isb / 255, iso];
        const u_InnerShadow = [offsetX, offsetY, blurRadius, 0];
        return [
            [
                ...u_Position,
                ...u_Size,
                ...u_FillColor,
                ...u_StrokeColor,
                ...u_ZIndexStrokeWidth,
                ...u_Opacity,
                ...u_InnerShadowColor,
                ...u_InnerShadow,
            ],
            {
                u_Position,
                u_Size,
                u_FillColor,
                u_StrokeColor,
                u_ZIndexStrokeWidth,
                u_Opacity,
                u_InnerShadowColor,
                u_InnerShadow,
            },
        ];
    }
}
exports.SDF = SDF;
_SDF_uniformBuffer = new WeakMap(), _SDF_texture = new WeakMap();
//# sourceMappingURL=SDF.js.map