"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var _SDFText_glyphManager, _SDFText_uniformBuffer, _SDFText_texture;
Object.defineProperty(exports, "__esModule", { value: true });
exports.SDFText = void 0;
const g_device_api_1 = require("@antv/g-device-api");
const Drawcall_1 = require("./Drawcall");
const sdf_text_1 = require("../shaders/sdf_text");
const utils_1 = require("../utils");
const components_1 = require("../components");
const systems_1 = require("../systems");
class SDFText extends Drawcall_1.Drawcall {
    constructor() {
        super(...arguments);
        _SDFText_glyphManager.set(this, new utils_1.GlyphManager());
        _SDFText_uniformBuffer.set(this, void 0);
        _SDFText_texture.set(this, void 0);
    }
    validate(shape) {
        const result = super.validate(shape);
        if (!result) {
            return false;
        }
        if (this.shapes.length === 0) {
            return true;
        }
        if (this.hash(this.shapes[0]) !== this.hash(shape)) {
            return false;
        }
        return true;
    }
    hash(shape) {
        const { bitmapFont, physical } = shape.read(components_1.Text);
        const font = shape.has(components_1.ComputedTextMetrics)
            ? shape.read(components_1.ComputedTextMetrics)
            : '';
        const blurRadius = shape.has(components_1.DropShadow)
            ? shape.read(components_1.DropShadow).blurRadius
            : 0;
        return `${font}-${bitmapFont === null || bitmapFont === void 0 ? void 0 : bitmapFont.fontFamily}-${physical}-${blurRadius}`;
    }
    get useBitmapFont() {
        return !!this.shapes[0].read(components_1.Text).bitmapFont;
    }
    createGeometry() {
        const { fontFamily, fontWeight, fontStyle, bitmapFont, esdt, content } = this.shapes[0].read(components_1.Text);
        const { font, fontMetrics } = this.shapes[0].read(components_1.ComputedTextMetrics);
        const { value: fill } = this.shapes[0].has(components_1.FillSolid)
            ? this.shapes[0].read(components_1.FillSolid)
            : { value: 'black' };
        const hasEmoji = (0, utils_1.containsEmoji)(content);
        const indices = [];
        const positions = [];
        const uvOffsets = [];
        let indicesOff = 0;
        let fontScale;
        if (this.useBitmapFont) {
            fontScale = fontMetrics.fontSize / bitmapFont.baseMeasurementFontSize;
        }
        else {
            // scale current font size to base(24)
            fontScale = fontMetrics.fontSize / (utils_1.BASE_FONT_WIDTH / utils_1.SDF_SCALE);
            const allText = this.shapes
                .map((text) => text.read(components_1.ComputedTextMetrics).bidiChars)
                .join('');
            __classPrivateFieldGet(this, _SDFText_glyphManager, "f").generateAtlas(font, fontFamily, fontWeight.toString(), fontStyle, allText, this.device, esdt, hasEmoji ? fill : '');
        }
        this.shapes.forEach((object) => {
            const metrics = object.read(components_1.ComputedTextMetrics);
            const { letterSpacing, bitmapFont } = object.read(components_1.Text);
            const { font, lines, lineHeight } = metrics;
            const { indicesOffset, indexBuffer, charUVOffsetBuffer, charPositionsBuffer, } = this.buildTextBuffers({
                object,
                lines,
                fontStack: font,
                lineHeight,
                letterSpacing: letterSpacing,
                indicesOffset: indicesOff,
                bitmapFont,
                fontScale,
            });
            indicesOff = indicesOffset;
            uvOffsets.push(...charUVOffsetBuffer);
            positions.push(...charPositionsBuffer);
            indices.push(...indexBuffer);
        });
        this.indexBufferData = new Uint32Array(indices);
        if (this.vertexBuffers[0]) {
            this.vertexBuffers[0].destroy();
        }
        this.vertexBufferDatas[0] = new Float32Array(positions);
        this.vertexBuffers[0] = this.device.createBuffer({
            viewOrSize: this.vertexBufferDatas[0],
            usage: g_device_api_1.BufferUsage.VERTEX,
            hint: g_device_api_1.BufferFrequencyHint.STATIC,
        });
        if (this.vertexBuffers[1]) {
            this.vertexBuffers[1].destroy();
        }
        this.vertexBufferDatas[1] = new Float32Array(uvOffsets);
        this.vertexBuffers[1] = this.device.createBuffer({
            viewOrSize: this.vertexBufferDatas[1],
            usage: g_device_api_1.BufferUsage.VERTEX,
            hint: g_device_api_1.BufferFrequencyHint.STATIC,
        });
        if (this.indexBuffer) {
            this.indexBuffer.destroy();
        }
        this.indexBuffer = this.device.createBuffer({
            viewOrSize: this.indexBufferData,
            usage: g_device_api_1.BufferUsage.INDEX,
            hint: g_device_api_1.BufferFrequencyHint.STATIC,
        });
        this.vertexBufferDescriptors = [
            {
                arrayStride: 4 * 3,
                stepMode: g_device_api_1.VertexStepMode.VERTEX,
                attributes: [
                    {
                        shaderLocation: sdf_text_1.Location.POSITION, // a_Position
                        offset: 0,
                        format: g_device_api_1.Format.F32_RGB,
                    },
                ],
            },
            {
                arrayStride: 4 * 4,
                stepMode: g_device_api_1.VertexStepMode.VERTEX,
                attributes: [
                    {
                        shaderLocation: sdf_text_1.Location.UV_OFFSET, // a_UvOffset
                        offset: 0,
                        format: g_device_api_1.Format.F32_RGBA,
                    },
                ],
            },
        ];
    }
    createMaterial(defines, uniformBuffer) {
        const { content, physical } = this.shapes[0].read(components_1.Text);
        const { blurRadius } = this.shapes[0].has(components_1.DropShadow)
            ? this.shapes[0].read(components_1.DropShadow)
            : { blurRadius: 0 };
        let glyphAtlasTexture;
        if (this.useBitmapFont) {
            const { bitmapFont } = this.shapes[0].read(components_1.Text);
            bitmapFont.createTexture(this.device);
            glyphAtlasTexture = bitmapFont.pages[0].texture;
            const { distanceField } = bitmapFont;
            if (distanceField.type === 'msdf') {
                defines += '#define USE_MSDF\n';
            }
            else if (distanceField.type === 'sdf') {
                defines += '#define USE_SDF\n';
            }
            else {
                defines += '#define USE_SDF_NONE\n';
            }
        }
        else {
            defines += '#define USE_SDF\n';
            const hasEmoji = (0, utils_1.containsEmoji)(content);
            if (hasEmoji) {
                defines += '#define USE_EMOJI\n';
            }
            glyphAtlasTexture = __classPrivateFieldGet(this, _SDFText_glyphManager, "f").getAtlasTexture();
        }
        if (blurRadius > 0) {
            defines += '#define USE_SHADOW\n';
        }
        this.device.setResourceName(glyphAtlasTexture, 'SDFText Texture');
        this.createProgram(sdf_text_1.vert, physical ? sdf_text_1.physical_frag : sdf_text_1.frag, defines);
        if (!__classPrivateFieldGet(this, _SDFText_uniformBuffer, "f")) {
            __classPrivateFieldSet(this, _SDFText_uniformBuffer, this.device.createBuffer({
                viewOrSize: Float32Array.BYTES_PER_ELEMENT * (16 + 4 + 4 + 4 + 4 + 4 + 4 + 4 + 4),
                usage: g_device_api_1.BufferUsage.UNIFORM,
                hint: g_device_api_1.BufferFrequencyHint.DYNAMIC,
            }), "f");
        }
        this.pipeline = this.renderCache.createRenderPipeline({
            inputLayout: this.inputLayout,
            program: this.program,
            colorAttachmentFormats: [g_device_api_1.Format.U8_RGBA_RT],
            depthStencilAttachmentFormat: g_device_api_1.Format.D24_S8,
            megaStateDescriptor: {
                attachmentsState: [
                    {
                        channelWriteMask: g_device_api_1.ChannelWriteMask.ALL,
                        rgbBlendState: {
                            blendMode: g_device_api_1.BlendMode.ADD,
                            blendSrcFactor: g_device_api_1.BlendFactor.SRC_ALPHA,
                            blendDstFactor: g_device_api_1.BlendFactor.ONE_MINUS_SRC_ALPHA,
                        },
                        alphaBlendState: {
                            blendMode: g_device_api_1.BlendMode.ADD,
                            blendSrcFactor: g_device_api_1.BlendFactor.ONE,
                            blendDstFactor: g_device_api_1.BlendFactor.ONE_MINUS_SRC_ALPHA,
                        },
                    },
                ],
                blendConstant: g_device_api_1.TransparentBlack,
                depthWrite: true,
                depthCompare: g_device_api_1.CompareFunction.GREATER,
                stencilWrite: false,
                stencilFront: {
                    compare: g_device_api_1.CompareFunction.ALWAYS,
                    passOp: g_device_api_1.StencilOp.KEEP,
                    failOp: g_device_api_1.StencilOp.KEEP,
                    depthFailOp: g_device_api_1.StencilOp.KEEP,
                },
                stencilBack: {
                    compare: g_device_api_1.CompareFunction.ALWAYS,
                    passOp: g_device_api_1.StencilOp.KEEP,
                    failOp: g_device_api_1.StencilOp.KEEP,
                    depthFailOp: g_device_api_1.StencilOp.KEEP,
                },
            },
        });
        this.device.setResourceName(this.pipeline, 'SDFTextPipeline');
        const sampler = this.renderCache.createSampler({
            addressModeU: g_device_api_1.AddressMode.CLAMP_TO_EDGE,
            addressModeV: g_device_api_1.AddressMode.CLAMP_TO_EDGE,
            minFilter: g_device_api_1.FilterMode.POINT,
            magFilter: g_device_api_1.FilterMode.BILINEAR,
            mipmapFilter: g_device_api_1.MipmapFilterMode.LINEAR,
            lodMinClamp: 0,
            lodMaxClamp: 0,
        });
        const bindings = {
            pipeline: this.pipeline,
            uniformBufferBindings: [
                {
                    buffer: uniformBuffer,
                },
                {
                    buffer: __classPrivateFieldGet(this, _SDFText_uniformBuffer, "f"),
                },
            ],
            samplerBindings: [
                {
                    texture: glyphAtlasTexture,
                    sampler,
                },
            ],
        };
        this.bindings = this.renderCache.createBindings(bindings);
    }
    render(renderPass, uniformLegacyObject) {
        // if (
        //   this.shapes.some((shape) => shape.renderDirtyFlag) ||
        //   this.geometryDirty
        // ) {
        const { matrix } = this.shapes[0].read(components_1.GlobalTransform);
        const u_ModelMatrix = [
            matrix.m00,
            matrix.m01,
            matrix.m02,
            matrix.m10,
            matrix.m11,
            matrix.m12,
            matrix.m20,
            matrix.m21,
            matrix.m22,
        ];
        const [buffer, legacyObject] = this.generateBuffer(this.shapes[0], this.useBitmapFont
            ? this.shapes[0].read(components_1.Text).bitmapFont.pages[0]
            : __classPrivateFieldGet(this, _SDFText_glyphManager, "f").getAtlas().image);
        __classPrivateFieldGet(this, _SDFText_uniformBuffer, "f").setSubData(0, new Uint8Array(new Float32Array([
            ...(0, utils_1.paddingMat3)(components_1.Mat3.fromGLMat3(u_ModelMatrix)),
            ...buffer,
        ]).buffer));
        this.program.setUniformsLegacy(Object.assign({ u_ModelMatrix }, legacyObject));
        if (this.useWireframe) {
            this.generateWireframe();
        }
        // }
        this.program.setUniformsLegacy(uniformLegacyObject);
        renderPass.setPipeline(this.pipeline);
        const vertexBuffers = this.vertexBuffers.map((buffer) => ({ buffer }));
        if (this.useWireframe) {
            vertexBuffers.push({ buffer: this.barycentricBuffer });
        }
        renderPass.setVertexInput(this.inputLayout, vertexBuffers, {
            buffer: this.indexBuffer,
        });
        renderPass.setBindings(this.bindings);
        renderPass.drawIndexed(this.indexBufferData.length);
    }
    destroy() {
        var _a, _b;
        super.destroy();
        if (this.program) {
            (_a = __classPrivateFieldGet(this, _SDFText_uniformBuffer, "f")) === null || _a === void 0 ? void 0 : _a.destroy();
            (_b = __classPrivateFieldGet(this, _SDFText_texture, "f")) === null || _b === void 0 ? void 0 : _b.destroy();
        }
        __classPrivateFieldGet(this, _SDFText_glyphManager, "f").destroy();
    }
    generateBuffer(shape, image) {
        const sizeAttenuation = 0;
        const globalRenderOrder = shape.has(components_1.GlobalRenderOrder)
            ? shape.read(components_1.GlobalRenderOrder).value
            : 0;
        const { value: fill } = shape.has(components_1.FillSolid)
            ? shape.read(components_1.FillSolid)
            : { value: null };
        const { r: fr, g: fg, b: fb, opacity: fo } = (0, utils_1.parseColor)(fill);
        const { opacity, strokeOpacity, fillOpacity } = shape.has(components_1.Opacity)
            ? shape.read(components_1.Opacity)
            : { opacity: 1, strokeOpacity: 1, fillOpacity: 1 };
        const { color: strokeColor, width } = shape.has(components_1.Stroke)
            ? shape.read(components_1.Stroke)
            : { color: null, width: 0 };
        const { r: sr, g: sg, b: sb, opacity: so } = (0, utils_1.parseColor)(strokeColor);
        const { color: dropShadowColor, offsetX, offsetY, blurRadius, } = shape.has(components_1.DropShadow)
            ? shape.read(components_1.DropShadow)
            : { color: null, offsetX: 0, offsetY: 0, blurRadius: 0 };
        const { r: dsR, g: dsG, b: dsB, opacity: dsO, } = (0, utils_1.parseColor)(dropShadowColor);
        const { width: atlasWidth, height: atlasHeight } = image;
        const { fontSize } = shape.read(components_1.ComputedTextMetrics).fontMetrics;
        const u_FillColor = [fr / 255, fg / 255, fb / 255, fo];
        const u_StrokeColor = [sr / 255, sg / 255, sb / 255, so];
        const u_ZIndexStrokeWidth = [
            globalRenderOrder / Drawcall_1.ZINDEX_FACTOR,
            width,
            fontSize,
            0,
        ];
        const u_Opacity = [
            opacity,
            fillOpacity,
            strokeOpacity,
            sizeAttenuation ? 1 : 0,
        ];
        const u_DropShadowColor = [dsR / 255, dsG / 255, dsB / 255, dsO];
        const u_DropShadow = [offsetX, offsetY, blurRadius, 0];
        const u_AtlasSize = [atlasWidth, atlasHeight];
        return [
            [
                ...u_FillColor,
                ...u_StrokeColor,
                ...u_ZIndexStrokeWidth,
                ...u_Opacity,
                ...u_DropShadowColor,
                ...u_DropShadow,
                ...u_AtlasSize,
            ],
            {
                u_FillColor,
                u_StrokeColor,
                u_ZIndexStrokeWidth,
                u_Opacity,
                u_DropShadowColor,
                u_DropShadow,
                u_AtlasSize,
            },
        ];
    }
    buildTextBuffers({ object, lines, fontStack, lineHeight, letterSpacing, indicesOffset, bitmapFont, fontScale, }) {
        const { textAlign, textBaseline, x = 0, y = 0, bitmapFontKerning, } = object.read(components_1.Text);
        const metrics = object.read(components_1.ComputedTextMetrics);
        const globalRenderOrder = object.has(components_1.GlobalRenderOrder)
            ? object.read(components_1.GlobalRenderOrder).value
            : 0;
        const charUVOffsetBuffer = [];
        const charPositionsBuffer = [];
        const indexBuffer = [];
        let i = indicesOffset;
        const dy = (0, systems_1.yOffsetFromTextBaseline)(textBaseline, metrics.fontMetrics) +
            metrics.fontMetrics.fontBoundingBoxAscent || 0;
        const positionedGlyphs = __classPrivateFieldGet(this, _SDFText_glyphManager, "f").layout(lines, fontStack, lineHeight, textAlign, letterSpacing, bitmapFont, fontScale, bitmapFontKerning, 0, dy);
        let positions;
        if (bitmapFont) {
            positions = {
                [fontStack]: Object.keys(bitmapFont.chars).reduce((acc, char) => {
                    const { xAdvance, xOffset, yOffset, rect } = bitmapFont.chars[char];
                    acc[char] = {
                        rect,
                        metrics: {
                            width: xAdvance,
                            height: bitmapFont.lineHeight,
                            left: xOffset,
                            top: -yOffset,
                            advance: xAdvance,
                        },
                    };
                    return acc;
                }, {}),
            };
        }
        else {
            // calculate position information for each individual character relative to the anchor point
            positions = __classPrivateFieldGet(this, _SDFText_glyphManager, "f").getAtlas().positions;
        }
        (0, utils_1.getGlyphQuads)(positionedGlyphs, positions, this.useBitmapFont).forEach((quad, index, total) => {
            // interleaved uv & offsets
            charUVOffsetBuffer.push(quad.tex.x, quad.tex.y, quad.tl.x, quad.tl.y);
            charUVOffsetBuffer.push(quad.tex.x + quad.tex.w, quad.tex.y, quad.tr.x, quad.tr.y);
            charUVOffsetBuffer.push(quad.tex.x + quad.tex.w, quad.tex.y + quad.tex.h, quad.br.x, quad.br.y);
            charUVOffsetBuffer.push(quad.tex.x, quad.tex.y + quad.tex.h, quad.bl.x, quad.bl.y);
            const zIndex = (globalRenderOrder + (1 / total.length) * index) / Drawcall_1.ZINDEX_FACTOR;
            charPositionsBuffer.push(x, y, zIndex, x, y, zIndex, x, y, zIndex, x, y, zIndex);
            indexBuffer.push(0 + i, 2 + i, 1 + i);
            indexBuffer.push(2 + i, 0 + i, 3 + i);
            i += 4;
        });
        return {
            indexBuffer,
            charUVOffsetBuffer,
            charPositionsBuffer,
            indicesOffset: i,
        };
    }
}
exports.SDFText = SDFText;
_SDFText_glyphManager = new WeakMap(), _SDFText_uniformBuffer = new WeakMap(), _SDFText_texture = new WeakMap();
//# sourceMappingURL=SDFText.js.map