"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _TexturePool_canvas, _TexturePool_ctx, _TexturePool_gradientCache, _TexturePool_patternCache;
Object.defineProperty(exports, "__esModule", { value: true });
exports.TexturePool = void 0;
exports.generateGradientKey = generateGradientKey;
exports.generatePatternKey = generatePatternKey;
const util_1 = require("@antv/util");
const environment_1 = require("../environment");
const utils_1 = require("../utils");
class TexturePool {
    constructor() {
        _TexturePool_canvas.set(this, void 0);
        _TexturePool_ctx.set(this, void 0);
        _TexturePool_gradientCache.set(this, {});
        _TexturePool_patternCache.set(this, {});
        __classPrivateFieldSet(this, _TexturePool_canvas, environment_1.DOMAdapter.get().createCanvas(128, 128), "f");
        __classPrivateFieldSet(this, _TexturePool_ctx, __classPrivateFieldGet(this, _TexturePool_canvas, "f").getContext('2d'), "f");
    }
    destroy() {
        __classPrivateFieldSet(this, _TexturePool_gradientCache, {}, "f");
        __classPrivateFieldSet(this, _TexturePool_patternCache, {}, "f");
    }
    getOrCreatePattern(params) {
        const { pattern, width, height } = params;
        const { image, repetition } = pattern;
        __classPrivateFieldGet(this, _TexturePool_canvas, "f").width = width;
        __classPrivateFieldGet(this, _TexturePool_canvas, "f").height = height;
        // TODO: load image
        if ((0, util_1.isString)(image)) {
            return __classPrivateFieldGet(this, _TexturePool_canvas, "f");
        }
        let canvasPattern = null;
        const key = generatePatternKey(params);
        if (__classPrivateFieldGet(this, _TexturePool_patternCache, "f")[key]) {
            canvasPattern = __classPrivateFieldGet(this, _TexturePool_patternCache, "f")[key];
        }
        else {
            canvasPattern = image && __classPrivateFieldGet(this, _TexturePool_ctx, "f").createPattern(image, repetition);
            __classPrivateFieldGet(this, _TexturePool_patternCache, "f")[key] = canvasPattern;
            // @see https://developer.mozilla.org/en-US/docs/Web/API/CanvasPattern/setTransform
            // if (transform) {
            //   const mat = parsedTransformToMat4(
            //     parseTransform(transform),
            //     new DisplayObject({}),
            //   );
            //   canvasPattern.setTransform({
            //     a: mat[0],
            //     b: mat[1],
            //     c: mat[4],
            //     d: mat[5],
            //     e: mat[12],
            //     f: mat[13],
            //   });
            // }
        }
        __classPrivateFieldGet(this, _TexturePool_ctx, "f").fillStyle = canvasPattern;
        __classPrivateFieldGet(this, _TexturePool_ctx, "f").fillRect(0, 0, width, height);
        return environment_1.DOMAdapter.get().createTexImageSource(__classPrivateFieldGet(this, _TexturePool_canvas, "f"));
    }
    getOrCreateGradient(params) {
        const { width, height, gradients } = params;
        __classPrivateFieldGet(this, _TexturePool_canvas, "f").width = width;
        __classPrivateFieldGet(this, _TexturePool_canvas, "f").height = height;
        gradients.forEach((g) => {
            const gradient = this.getOrCreateGradientInternal(Object.assign(Object.assign({}, g), { width,
                height, min: [0, 0] }));
            __classPrivateFieldGet(this, _TexturePool_ctx, "f").fillStyle = gradient;
            __classPrivateFieldGet(this, _TexturePool_ctx, "f").fillRect(0, 0, width, height);
        });
        return environment_1.DOMAdapter.get().createTexImageSource(__classPrivateFieldGet(this, _TexturePool_canvas, "f"));
    }
    getOrCreateGradientInternal(params) {
        const key = generateGradientKey(params);
        const { type, steps, min, width, height } = params;
        if (__classPrivateFieldGet(this, _TexturePool_gradientCache, "f")[key]) {
            return __classPrivateFieldGet(this, _TexturePool_gradientCache, "f")[key];
        }
        let gradient = null;
        if (type === 'linear-gradient') {
            const { x1, y1, x2, y2 } = (0, utils_1.computeLinearGradient)(min, width, height, params.angle);
            // @see https://developer.mozilla.org/zh-CN/docs/Web/API/CanvasRenderingContext2D/createLinearGradient
            gradient = __classPrivateFieldGet(this, _TexturePool_ctx, "f").createLinearGradient(x1, y1, x2, y2);
        }
        else if (type === 'radial-gradient') {
            const { cx, cy, size } = params;
            const { x, y, r } = (0, utils_1.computeRadialGradient)(min, width, height, cx, cy, size);
            // @see https://developer.mozilla.org/zh-CN/docs/Web/API/CanvasRenderingContext2D/createRadialGradient
            gradient = __classPrivateFieldGet(this, _TexturePool_ctx, "f").createRadialGradient(x, y, 0, x, y, r);
        }
        else if (type === 'conic-gradient') {
            const { cx, cy, angle } = params;
            const { x, y } = (0, utils_1.computeConicGradient)(min, width, height, cx, cy);
            gradient = __classPrivateFieldGet(this, _TexturePool_ctx, "f").createConicGradient(angle, x, y);
        }
        if (gradient) {
            steps.forEach(({ offset, color }) => {
                if (offset.type === '%') {
                    gradient === null || gradient === void 0 ? void 0 : gradient.addColorStop(offset.value / 100, color.toString());
                }
            });
            __classPrivateFieldGet(this, _TexturePool_gradientCache, "f")[key] = gradient;
        }
        return __classPrivateFieldGet(this, _TexturePool_gradientCache, "f")[key];
    }
}
exports.TexturePool = TexturePool;
_TexturePool_canvas = new WeakMap(), _TexturePool_ctx = new WeakMap(), _TexturePool_gradientCache = new WeakMap(), _TexturePool_patternCache = new WeakMap();
function generateGradientKey(params) {
    const { type, min, width, height, steps } = params;
    const suffix = `${Math.round(min[0])}-${Math.round(min[1])}-${Math.round(width)}-${Math.round(height)}-${steps
        .map(({ offset, color }) => `${offset.value}${color}`)
        .join('-')}`;
    if (type === 'linear-gradient') {
        const { angle } = params;
        return `${type}-${(0, utils_1.hashCode)(`${Math.round(angle)}-${suffix}`)}`;
    }
    else if (type === 'radial-gradient') {
        const { cx, cy, size } = params;
        return `${type}-${(0, utils_1.hashCode)(`${Math.round(cx.value)}-${Math.round(cy.value)}-${Math.round(Number((size === null || size === void 0 ? void 0 : size.value) || 0))}-${suffix}`)}`;
    }
    else if (type === 'conic-gradient') {
        const { cx, cy, angle } = params;
        return `${type}-${(0, utils_1.hashCode)(`${Math.round(cx.value)}-${Math.round(cy.value)}-${Math.round(angle || 0)}-${suffix}`)}`;
    }
}
function generatePatternKey(params) {
    const { image, repetition, transform } = params.pattern;
    // TODO: when image is not string
    return `pattern-${(0, utils_1.hashCode)(`${image}-${repetition}-${transform}`)}`;
}
//# sourceMappingURL=TexturePool.js.map