"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var _BatchManager_drawcallsToFlush, _BatchManager_nonBatchableDrawcallsCache, _BatchManager_batchableDrawcallsCache, _BatchManager_instancesCache;
Object.defineProperty(exports, "__esModule", { value: true });
exports.BatchManager = void 0;
const drawcalls_1 = require("../drawcalls");
const components_1 = require("../components");
/**
 * Since a shape may have multiple drawcalls, we need to cache them and maintain an 1-to-many relationship.
 *
 * e.g. we need 2 drawcalls for a Circle with dashed stroke:
 * - A SDF drawcall to draw the fill.
 * - A Path drawcall to draw the dashed stroke.
 *
 * e.g. 3 drawcalls for a Rect with drop shadow.
 */
function getDrawcallCtors(shape) {
    const SHAPE_DRAWCALL_CTORS = [];
    if (shape.has(components_1.Circle) || shape.has(components_1.Ellipse)) {
        if (shape.has(components_1.Rough)) {
            SHAPE_DRAWCALL_CTORS.push(drawcalls_1.Mesh, drawcalls_1.SmoothPolyline, drawcalls_1.SmoothPolyline);
        }
        else {
            SHAPE_DRAWCALL_CTORS.push(drawcalls_1.SDF, drawcalls_1.SmoothPolyline);
        }
    }
    else if (shape.has(components_1.Rect)) {
        if (shape.has(components_1.Rough)) {
            SHAPE_DRAWCALL_CTORS.push(drawcalls_1.ShadowRect, drawcalls_1.Mesh, drawcalls_1.SmoothPolyline, drawcalls_1.SmoothPolyline);
        }
        else {
            SHAPE_DRAWCALL_CTORS.push(drawcalls_1.ShadowRect, drawcalls_1.SDF, drawcalls_1.SmoothPolyline);
        }
    }
    else if (shape.has(components_1.Polyline)) {
        SHAPE_DRAWCALL_CTORS.push(drawcalls_1.SmoothPolyline);
    }
    else if (shape.has(components_1.Path)) {
        if (shape.has(components_1.Rough)) {
            SHAPE_DRAWCALL_CTORS.push(drawcalls_1.Mesh, drawcalls_1.SmoothPolyline, drawcalls_1.SmoothPolyline);
        }
        else {
            SHAPE_DRAWCALL_CTORS.push(drawcalls_1.Mesh, drawcalls_1.SmoothPolyline);
        }
    }
    else if (shape.has(components_1.Text)) {
        SHAPE_DRAWCALL_CTORS.push(drawcalls_1.SDFText);
    }
    return SHAPE_DRAWCALL_CTORS;
}
// SHAPE_DRAWCALL_CTORS.set(Custom, [CustomDrawcall]);
class BatchManager {
    constructor(device, renderCache, texturePool) {
        this.device = device;
        this.renderCache = renderCache;
        this.texturePool = texturePool;
        /**
         * Drawcalls to flush in the next frame.
         */
        _BatchManager_drawcallsToFlush.set(this, []);
        /**
         * Cache drawcalls for non batchable shape.
         */
        _BatchManager_nonBatchableDrawcallsCache.set(this, Object.create(null));
        _BatchManager_batchableDrawcallsCache.set(this, Object.create(null));
        _BatchManager_instancesCache.set(this, Object.create(null));
    }
    collectDrawcallCtors(shape) {
        return getDrawcallCtors(shape)
            .map((DrawcallCtor) => {
            if (
            // @ts-ignore
            !DrawcallCtor.check ||
                // @ts-ignore
                (DrawcallCtor.check && DrawcallCtor.check(shape))) {
                return DrawcallCtor;
            }
        })
            .filter((drawcallCtor) => !!drawcallCtor);
    }
    createDrawcalls(shape, instanced = false) {
        return this.collectDrawcallCtors(shape).map((DrawcallCtor, index) => {
            // @ts-ignore
            const drawcall = new DrawcallCtor(this.device, this.renderCache, this.texturePool, instanced, index);
            drawcall.add(shape);
            return drawcall;
        });
    }
    getOrCreateNonBatchableDrawcalls(shape) {
        let existed = __classPrivateFieldGet(this, _BatchManager_nonBatchableDrawcallsCache, "f")[shape.__id];
        if (!existed) {
            existed = this.createDrawcalls(shape) || [];
            __classPrivateFieldGet(this, _BatchManager_nonBatchableDrawcallsCache, "f")[shape.__id] = existed;
        }
        return existed;
    }
    getOrCreateBatchableDrawcalls(shape) {
        let existed = __classPrivateFieldGet(this, _BatchManager_batchableDrawcallsCache, "f")[shape.__id];
        if (!existed) {
            const geometryCtor = shape.has(components_1.Circle)
                ? shape.has(components_1.Rough)
                    ? 'rough-circle'
                    : 'circle'
                : shape.has(components_1.Ellipse)
                    ? shape.has(components_1.Rough)
                        ? 'rough-ellipse'
                        : 'ellipse'
                    : shape.has(components_1.Rect)
                        ? shape.has(components_1.Rough)
                            ? 'rough-rect'
                            : 'rect'
                        : shape.has(components_1.Polyline)
                            ? shape.has(components_1.Rough)
                                ? 'rough-polyline'
                                : 'polyline'
                            : shape.has(components_1.Path)
                                ? shape.has(components_1.Rough)
                                    ? 'rough-path'
                                    : 'path'
                                : shape.has(components_1.Text)
                                    ? 'text'
                                    : undefined;
            let instancedDrawcalls = __classPrivateFieldGet(this, _BatchManager_instancesCache, "f")[geometryCtor];
            if (!instancedDrawcalls) {
                instancedDrawcalls = [];
            }
            const ctors = this.collectDrawcallCtors(shape);
            existed = instancedDrawcalls.find((drawcalls) => drawcalls.length === ctors.length &&
                drawcalls.every((drawcall) => drawcall.validate(shape)));
            if (!existed) {
                existed = this.createDrawcalls(shape, true) || [];
                instancedDrawcalls.push(existed);
                __classPrivateFieldGet(this, _BatchManager_instancesCache, "f")[geometryCtor] = instancedDrawcalls;
            }
            existed.forEach((drawcall) => {
                drawcall.add(shape);
            });
            __classPrivateFieldGet(this, _BatchManager_batchableDrawcallsCache, "f")[shape.__id] = existed;
        }
        return existed;
    }
    add(shape) {
        if (shape.read(components_1.Renderable).batchable) {
            const drawcalls = this.getOrCreateBatchableDrawcalls(shape);
            if (__classPrivateFieldGet(this, _BatchManager_drawcallsToFlush, "f").indexOf(drawcalls[0]) === -1) {
                __classPrivateFieldGet(this, _BatchManager_drawcallsToFlush, "f").push(...drawcalls);
            }
        }
        else {
            __classPrivateFieldGet(this, _BatchManager_drawcallsToFlush, "f").push(...this.getOrCreateNonBatchableDrawcalls(shape));
        }
    }
    /**
     * Called when a shape is:
     * * removed from the scene graph.
     * * culled from viewport.
     * * invisible.
     */
    remove(shape) {
        if (shape.read(components_1.Renderable).batchable) {
            this.getOrCreateBatchableDrawcalls(shape).forEach((drawcall) => {
                drawcall.remove(shape);
            });
            delete __classPrivateFieldGet(this, _BatchManager_batchableDrawcallsCache, "f")[shape.__id];
        }
        else {
            delete __classPrivateFieldGet(this, _BatchManager_nonBatchableDrawcallsCache, "f")[shape.__id];
        }
    }
    destroy() {
        for (const key in __classPrivateFieldGet(this, _BatchManager_nonBatchableDrawcallsCache, "f")) {
            __classPrivateFieldGet(this, _BatchManager_nonBatchableDrawcallsCache, "f")[key].forEach((drawcall) => {
                if (!drawcall.destroyed) {
                    drawcall.destroy();
                }
            });
        }
        for (const key in __classPrivateFieldGet(this, _BatchManager_batchableDrawcallsCache, "f")) {
            __classPrivateFieldGet(this, _BatchManager_batchableDrawcallsCache, "f")[key].forEach((drawcall) => {
                if (!drawcall.destroyed) {
                    drawcall.destroy();
                }
            });
        }
    }
    clear() {
        __classPrivateFieldSet(this, _BatchManager_drawcallsToFlush, [], "f");
    }
    flush(renderPass, uniformBuffer, uniformLegacyObject) {
        const geometryDirtyDrawcalls = [];
        const materialDirtyDrawcalls = [];
        const geometryDirtyShapes = [];
        const materialDirtyShapes = [];
        __classPrivateFieldGet(this, _BatchManager_drawcallsToFlush, "f").forEach((drawcall) => {
            drawcall.shapes.forEach((shape) => {
                // if (shape.geometryDirtyFlag) {
                geometryDirtyShapes.push(shape);
                geometryDirtyDrawcalls.push(drawcall);
                // }
                // if (shape.materialDirtyFlag) {
                materialDirtyShapes.push(shape);
                materialDirtyDrawcalls.push(drawcall);
                // }
            });
        });
        geometryDirtyDrawcalls.forEach((drawcall) => (drawcall.geometryDirty = true));
        materialDirtyDrawcalls.forEach((drawcall) => (drawcall.materialDirty = true));
        // geometryDirtyShapes.forEach((shape) => {
        //   if (shape.geometryDirtyFlag) {
        //     shape.preCreateGeometry?.();
        //     shape.geometryDirtyFlag = false;
        //   }
        // });
        // materialDirtyShapes.forEach((shape) => (shape.materialDirtyFlag = false));
        __classPrivateFieldGet(this, _BatchManager_drawcallsToFlush, "f").forEach((drawcall) => {
            drawcall.submit(renderPass, uniformBuffer, uniformLegacyObject);
        });
    }
    stats() {
        return {
            drawcallCount: __classPrivateFieldGet(this, _BatchManager_drawcallsToFlush, "f").length,
        };
    }
}
exports.BatchManager = BatchManager;
_BatchManager_drawcallsToFlush = new WeakMap(), _BatchManager_nonBatchableDrawcallsCache = new WeakMap(), _BatchManager_batchableDrawcallsCache = new WeakMap(), _BatchManager_instancesCache = new WeakMap();
//# sourceMappingURL=BatchManager.js.map