"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComputeBounds = void 0;
const becsy_1 = require("@lastolivegames/becsy");
const components_1 = require("../components");
class ComputeBounds extends becsy_1.System {
    constructor() {
        super();
        this.renderables = this.query((q) => q.addedOrChanged
            .with(components_1.Renderable, components_1.GlobalTransform)
            .withAny(components_1.Circle, components_1.Ellipse, components_1.Rect, components_1.Polyline, components_1.Path, components_1.Text).trackWrites);
        this.query((q) => q.current.with(components_1.ComputedBounds).write);
        this.query((q) => q.using(components_1.Circle, components_1.Ellipse, components_1.Rect, components_1.Polyline, components_1.Path, components_1.ComputedPoints, components_1.Text, components_1.ComputedTextMetrics, components_1.Stroke, components_1.DropShadow).read);
    }
    execute() {
        this.renderables.addedOrChanged.forEach((entity) => {
            if (!entity.has(components_1.ComputedBounds)) {
                entity.add(components_1.ComputedBounds);
            }
            const stroke = entity.has(components_1.Stroke) ? entity.read(components_1.Stroke) : undefined;
            const dropShadow = entity.has(components_1.DropShadow)
                ? entity.read(components_1.DropShadow)
                : undefined;
            let geometryBounds;
            let renderBounds;
            if (entity.has(components_1.Circle)) {
                geometryBounds = components_1.Circle.getGeometryBounds(entity.read(components_1.Circle));
                renderBounds = components_1.Circle.getRenderBounds(entity.read(components_1.Circle), stroke);
            }
            else if (entity.has(components_1.Ellipse)) {
                geometryBounds = components_1.Ellipse.getGeometryBounds(entity.read(components_1.Ellipse));
                renderBounds = components_1.Ellipse.getRenderBounds(entity.read(components_1.Ellipse), stroke);
            }
            else if (entity.has(components_1.Rect)) {
                geometryBounds = components_1.Rect.getGeometryBounds(entity.read(components_1.Rect));
                renderBounds = components_1.Rect.getRenderBounds(entity.read(components_1.Rect), stroke, dropShadow);
            }
            else if (entity.has(components_1.Polyline)) {
                geometryBounds = components_1.Polyline.getGeometryBounds(Object.assign(Object.assign({}, entity.read(components_1.Polyline)), { points: entity.read(components_1.ComputedPoints).shiftedPoints }));
                renderBounds = components_1.Polyline.getRenderBounds(entity.read(components_1.Polyline), stroke);
            }
            else if (entity.has(components_1.Path)) {
                geometryBounds = components_1.Path.getGeometryBounds(entity.read(components_1.Path), entity.read(components_1.ComputedPoints));
                renderBounds = components_1.Path.getRenderBounds(entity.read(components_1.Path), entity.read(components_1.ComputedPoints), stroke);
            }
            else if (entity.has(components_1.Text)) {
                geometryBounds = components_1.Text.getGeometryBounds(entity.read(components_1.Text), entity.read(components_1.ComputedTextMetrics));
                renderBounds = components_1.Text.getRenderBounds(entity.read(components_1.Text), entity.read(components_1.ComputedTextMetrics), stroke, dropShadow);
            }
            if (geometryBounds) {
                entity.write(components_1.ComputedBounds).geometryBounds = geometryBounds;
            }
            if (renderBounds) {
                entity.write(components_1.ComputedBounds).renderBounds = renderBounds;
            }
        });
    }
}
exports.ComputeBounds = ComputeBounds;
//# sourceMappingURL=ComputeBounds.js.map