"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComputeRough = void 0;
const becsy_1 = require("@lastolivegames/becsy");
const components_1 = require("../components");
const utils_1 = require("../utils");
class ComputeRough extends becsy_1.System {
    constructor() {
        super();
        this.roughs = this.query((q) => q.addedOrChanged.with(components_1.Rough).trackWrites);
        this.query((q) => q.current.with(components_1.ComputedRough).write);
        this.query((q) => q.using(components_1.Circle, components_1.Ellipse, components_1.Rect, components_1.Polyline, components_1.Path, components_1.FillSolid, components_1.Stroke).read);
    }
    execute() {
        this.roughs.addedOrChanged.forEach((entity) => {
            let drawable;
            const roughOptions = (0, components_1.getRoughOptions)(entity);
            if (entity.has(components_1.Circle)) {
                const { cx, cy, r } = entity.read(components_1.Circle);
                drawable = utils_1.generator.circle(cx, cy, r * 2, roughOptions);
            }
            else if (entity.has(components_1.Ellipse)) {
                const { cx, cy, rx, ry } = entity.read(components_1.Ellipse);
                drawable = utils_1.generator.ellipse(cx, cy, rx * 2, ry * 2, roughOptions);
            }
            else if (entity.has(components_1.Rect)) {
                const { x, y, width, height } = entity.read(components_1.Rect);
                drawable = utils_1.generator.rectangle(x, y, width, height, roughOptions);
            }
            else if (entity.has(components_1.Polyline)) {
                const { points } = entity.read(components_1.Polyline);
                drawable = utils_1.generator.linearPath(points, roughOptions);
            }
            else if (entity.has(components_1.Path)) {
                const { d } = entity.read(components_1.Path);
                drawable = utils_1.generator.path(d, roughOptions);
            }
            const drawableSets = drawable.sets;
            let strokePoints = [];
            let fillPoints = [];
            let fillPathPoints = [];
            drawableSets.forEach((set) => {
                const { subPaths } = (0, utils_1.parsePath)(set);
                const points = subPaths.map((subPath) => subPath
                    .getPoints()
                    .map((point) => [point[0], point[1]]));
                if (set.type === 'path') {
                    strokePoints = points;
                }
                else if (set.type === 'fillPath') {
                    fillPathPoints = points;
                }
                else if (set.type === 'fillSketch') {
                    fillPoints = points;
                }
            });
            if (!entity.has(components_1.ComputedRough)) {
                entity.add(components_1.ComputedRough);
            }
            Object.assign(entity.write(components_1.ComputedRough), {
                drawableSets,
                strokePoints,
                fillPoints,
                fillPathPoints,
            });
        });
    }
}
exports.ComputeRough = ComputeRough;
//# sourceMappingURL=ComputeRough.js.map