"use strict";
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _ComputeTextMetrics_canvas, _ComputeTextMetrics_context, _ComputeTextMetrics_bidi, _ComputeTextMetrics_bidiCache, _ComputeTextMetrics_fonts;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ComputeTextMetrics = void 0;
exports.fontStringFromTextStyle = fontStringFromTextStyle;
exports.yOffsetFromTextBaseline = yOffsetFromTextBaseline;
const becsy_1 = require("@lastolivegames/becsy");
const math_1 = require("@pixi/math");
const bidi_js_1 = __importDefault(require("bidi-js"));
const components_1 = require("../components");
const environment_1 = require("../environment");
const METRICS_STRING = '|ÉqÅ';
const BASELINE_SYMBOL = 'M';
const NEWLINES = [
    0x000a, // line feed
    0x000d, // carriage return
];
const BREAKING_SPACES = [
    0x0009, // character tabulation
    0x0020, // space
    0x2000, // en quad
    0x2001, // em quad
    0x2002, // en space
    0x2003, // em space
    0x2004, // three-per-em space
    0x2005, // four-per-em space
    0x2006, // six-per-em space
    0x2008, // punctuation space
    0x2009, // thin space
    0x200a, // hair space
    0x205f, // medium mathematical space
    0x3000, // ideographic space
];
const LATIN_REGEX = /[a-zA-Z0-9\u00C0-\u00D6\u00D8-\u00f6\u00f8-\u00ff!"#$%&'()*+,-./:;]/;
// Line breaking rules in CJK (Kinsoku Shori)
// Refer from https://en.wikipedia.org/wiki/Line_breaking_rules_in_East_Asian_languages
const regexCannotStartZhCn = /[!%),.:;?\]}¢°·'""†‡›℃∶、。〃〆〕〗〞﹚﹜！＂％＇），．：；？！］｝～]/;
const regexCannotEndZhCn = /[$(£¥·'"〈《「『【〔〖〝﹙﹛＄（．［｛￡￥]/;
const regexCannotStartZhTw = /[!),.:;?\]}¢·–—'"•"、。〆〞〕〉》」︰︱︲︳﹐﹑﹒﹓﹔﹕﹖﹘﹚﹜！），．：；？︶︸︺︼︾﹀﹂﹗］｜｝､]/;
const regexCannotEndZhTw = /[([{£¥'"‵〈《「『〔〝︴﹙﹛（｛︵︷︹︻︽︿﹁﹃﹏]/;
const regexCannotStartJaJp = /[)\]｝〕〉》」』】〙〗〟'"｠»ヽヾーァィゥェォッャュョヮヵヶぁぃぅぇぉっゃゅょゎゕゖㇰㇱㇲㇳㇴㇵㇶㇷㇸㇹㇺㇻㇼㇽㇾㇿ々〻‐゠–〜?!‼⁇⁈⁉・、:;,。.]/;
const regexCannotEndJaJp = /[([｛〔〈《「『【〘〖〝'"｟«—...‥〳〴〵]/;
const regexCannotStartKoKr = /[!%),.:;?\]}¢°'"†‡℃〆〈《「『〕！％），．：；？］｝]/;
const regexCannotEndKoKr = /[$([{£¥'"々〇〉》」〔＄（［｛｠￥￦#]/;
const regexCannotStart = new RegExp(`${regexCannotStartZhCn.source}|${regexCannotStartZhTw.source}|${regexCannotStartJaJp.source}|${regexCannotStartKoKr.source}`);
const regexCannotEnd = new RegExp(`${regexCannotEndZhCn.source}|${regexCannotEndZhTw.source}|${regexCannotEndJaJp.source}|${regexCannotEndKoKr.source}`);
// @see https://github.com/pixijs/pixijs/blob/dev/src/scene/text/canvas/utils/fontStringFromTextStyle.ts#L17
const genericFontFamilies = [
    'serif',
    'sans-serif',
    'monospace',
    'cursive',
    'fantasy',
    'system-ui',
];
/**
 * Generates a font style string to use for `TextMetrics.measureFont()`.
 * @param style
 * @returns Font style string, for passing to `TextMetrics.measureFont()`
 */
function fontStringFromTextStyle(style) {
    // build canvas api font setting from individual components. Convert a numeric style.fontSize to px
    const fontSizeString = typeof style.fontSize === 'number' ? `${style.fontSize}px` : style.fontSize;
    // Clean-up fontFamily property by quoting each font name
    // this will support font names with spaces
    let fontFamilies = style.fontFamily;
    if (!Array.isArray(style.fontFamily)) {
        fontFamilies = style.fontFamily.split(',');
    }
    for (let i = fontFamilies.length - 1; i >= 0; i--) {
        // Trim any extra white-space
        let fontFamily = fontFamilies[i].trim();
        // Check if font already contains strings
        if (!/([\"\'])[^\'\"]+\1/.test(fontFamily) &&
            !genericFontFamilies.includes(fontFamily)) {
            fontFamily = `"${fontFamily}"`;
        }
        fontFamilies[i] = fontFamily;
    }
    // eslint-disable-next-line max-len
    return `${style.fontStyle} ${style.fontVariant} ${style.fontWeight} ${fontSizeString} ${fontFamilies.join(',')}`;
}
function yOffsetFromTextBaseline(textBaseline, fontMetrics) {
    let offset = 0;
    const { fontBoundingBoxAscent = 0, fontBoundingBoxDescent = 0, hangingBaseline = 0, ideographicBaseline = 0, } = fontMetrics;
    if (textBaseline === 'alphabetic') {
        offset -= fontBoundingBoxAscent;
    }
    else if (textBaseline === 'middle') {
        offset -= (fontBoundingBoxAscent + fontBoundingBoxDescent) / 2;
    }
    else if (textBaseline === 'hanging') {
        offset -= hangingBaseline;
    }
    else if (textBaseline === 'ideographic') {
        offset -= ideographicBaseline;
    }
    else if (textBaseline === 'bottom') {
        offset -= fontBoundingBoxAscent + fontBoundingBoxDescent;
    }
    else if (textBaseline === 'top') {
        offset = 0;
    }
    return offset;
}
class ComputeTextMetrics extends becsy_1.System {
    constructor() {
        super();
        this.texts = this.query((q) => q.addedOrChanged.with(components_1.Text).trackWrites);
        _ComputeTextMetrics_canvas.set(this, void 0);
        _ComputeTextMetrics_context.set(this, void 0);
        _ComputeTextMetrics_bidi.set(this, (0, bidi_js_1.default)());
        _ComputeTextMetrics_bidiCache.set(this, {});
        _ComputeTextMetrics_fonts.set(this, {});
        this.shouldBreakByKinsokuShorui = (char, nextChar) => {
            if (this.isBreakingSpace(nextChar))
                return false;
            if (char) {
                // Line breaking rules in CJK (Kinsoku Shori)
                if (regexCannotEnd.exec(nextChar) || regexCannotStart.exec(char)) {
                    return true;
                }
            }
            return false;
        };
        this.trimByKinsokuShorui = (prev) => {
            const next = [...prev];
            const prevLine = next[next.length - 2];
            if (!prevLine) {
                return prev;
            }
            const lastChar = prevLine[prevLine.length - 1];
            next[next.length - 2] = prevLine.slice(0, -1);
            next[next.length - 1] = lastChar + next[next.length - 1];
            return next;
        };
        this.query((q) => q.current.with(components_1.ComputedTextMetrics).write);
        const canvas = environment_1.DOMAdapter.get().createCanvas(1, 1);
        if (canvas) {
            __classPrivateFieldSet(this, _ComputeTextMetrics_canvas, canvas, "f");
            __classPrivateFieldSet(this, _ComputeTextMetrics_context, canvas.getContext('2d'), "f");
        }
    }
    execute() {
        this.texts.addedOrChanged.forEach((entity) => {
            const text = entity.read(components_1.Text);
            const bidiChars = this.computeBidi(text.content);
            const metrics = this.measureText(text);
            if (!entity.has(components_1.ComputedTextMetrics)) {
                entity.add(components_1.ComputedTextMetrics);
            }
            Object.assign(entity.write(components_1.ComputedTextMetrics), Object.assign({ bidiChars }, metrics));
        });
    }
    measureFont(font) {
        if (__classPrivateFieldGet(this, _ComputeTextMetrics_fonts, "f")[font]) {
            return __classPrivateFieldGet(this, _ComputeTextMetrics_fonts, "f")[font];
        }
        __classPrivateFieldGet(this, _ComputeTextMetrics_context, "f").font = font;
        const { actualBoundingBoxAscent, actualBoundingBoxDescent, actualBoundingBoxLeft, actualBoundingBoxRight, alphabeticBaseline, fontBoundingBoxAscent, fontBoundingBoxDescent, hangingBaseline, ideographicBaseline, width, emHeightAscent, emHeightDescent, } = __classPrivateFieldGet(this, _ComputeTextMetrics_context, "f").measureText(METRICS_STRING + BASELINE_SYMBOL);
        const properties = {
            actualBoundingBoxAscent,
            actualBoundingBoxDescent,
            actualBoundingBoxLeft,
            actualBoundingBoxRight,
            alphabeticBaseline,
            fontBoundingBoxAscent,
            fontBoundingBoxDescent,
            hangingBaseline,
            ideographicBaseline,
            emHeightAscent,
            emHeightDescent,
            width,
            fontSize: actualBoundingBoxAscent + actualBoundingBoxDescent,
        };
        __classPrivateFieldGet(this, _ComputeTextMetrics_fonts, "f")[font] = properties;
        return properties;
    }
    measureText(style) {
        const { content } = style;
        const bidiChars = __classPrivateFieldGet(this, _ComputeTextMetrics_bidiCache, "f")[content];
        const { wordWrap, letterSpacing, textAlign, textBaseline, leading, bitmapFont, bitmapFontKerning, fontSize, } = style;
        // TODO: strokeWidth
        const strokeWidth = 0;
        let lineHeight = style.lineHeight;
        const font = fontStringFromTextStyle(style);
        let fontMetrics;
        let scale = 1;
        if (bitmapFont) {
            const textMetrics = this.measureBitmapFont(bitmapFont, style.fontSize);
            lineHeight = textMetrics.lineHeight;
            fontMetrics = textMetrics.fontMetrics;
            scale = textMetrics.scale;
        }
        else {
            fontMetrics = this.measureFont(font);
            __classPrivateFieldGet(this, _ComputeTextMetrics_context, "f").font = font;
        }
        lineHeight *= scale;
        // fallback in case UA disallow canvas data extraction
        if (fontMetrics.fontSize === 0) {
            fontMetrics.fontSize = fontSize;
        }
        const outputText = wordWrap
            ? this.wordWrap(bidiChars, style, scale)
            : bidiChars;
        const lines = outputText.split(/(?:\r\n|\r|\n)/);
        const lineWidths = new Array(lines.length);
        let maxLineWidth = 0;
        for (let i = 0; i < lines.length; i++) {
            const lineWidth = this.measureTextInternal(lines[i], letterSpacing, bitmapFont, bitmapFontKerning, scale);
            lineWidths[i] = lineWidth;
            maxLineWidth = Math.max(maxLineWidth, lineWidth);
        }
        // const {
        //   fontBoundingBoxAscent,
        //   fontBoundingBoxDescent,
        //   hangingBaseline,
        //   ideographicBaseline,
        // } = fontMetrics;
        const width = maxLineWidth + strokeWidth;
        lineHeight = lineHeight || fontMetrics.fontSize + strokeWidth;
        const height = Math.max(lineHeight, fontMetrics.fontSize + strokeWidth) +
            (lines.length - 1) * (lineHeight + leading);
        lineHeight += leading;
        // handle vertical text baseline
        let offsetY = 0;
        if (textBaseline === 'middle') {
            offsetY = -height / 2;
        }
        else if (textBaseline === 'bottom' ||
            textBaseline === 'alphabetic' ||
            textBaseline === 'ideographic') {
            offsetY = -height;
        }
        else if (textBaseline === 'top' || textBaseline === 'hanging') {
            offsetY = 0;
        }
        return {
            font,
            width,
            height,
            lines,
            lineWidths,
            lineHeight,
            maxLineWidth,
            fontMetrics,
            lineMetrics: lineWidths.map((width, i) => {
                let offsetX = 0;
                // handle horizontal text align
                if (textAlign === 'center') {
                    offsetX -= width / 2;
                }
                else if (textAlign === 'right' || textAlign === 'end') {
                    offsetX -= width;
                }
                return new math_1.Rectangle(offsetX - strokeWidth / 2, offsetY + i * lineHeight, width + strokeWidth, lineHeight);
            }),
        };
    }
    /**
     * @see https://github.com/pixijs/pixijs/blob/dev/src/scene/text/canvas/CanvasTextMetrics.ts#L369
     */
    wordWrap(text, style, scale) {
        const context = __classPrivateFieldGet(this, _ComputeTextMetrics_canvas, "f").getContext('2d', {
            willReadFrequently: true,
        });
        const { letterSpacing, textOverflow, maxLines, bitmapFont } = style;
        // How to handle whitespaces
        // const collapseSpaces = this.collapseSpaces(whiteSpace);
        // const collapseNewlines = this.collapseNewlines(whiteSpace);
        // whether or not spaces may be added to the beginning of lines
        // let canPrependSpaces = !collapseSpaces;
        // There is letterSpacing after every char except the last one
        // t_h_i_s_' '_i_s_' '_a_n_' '_e_x_a_m_p_l_e_' '_!
        // so for convenience the above needs to be compared to width + 1 extra letterSpace
        // t_h_i_s_' '_i_s_' '_a_n_' '_e_x_a_m_p_l_e_' '_!_
        // ________________________________________________
        // And then the final space is simply no appended to each line
        const maxWidth = style.wordWrapWidth + letterSpacing;
        let ellipsis = '';
        if (textOverflow === 'ellipsis') {
            ellipsis = '...';
        }
        else if (textOverflow && textOverflow !== 'clip') {
            ellipsis = textOverflow;
        }
        let lines = [];
        let currentIndex = 0;
        let currentWidth = 0;
        const cache = {};
        const calcWidth = (char) => {
            return this.getFromCache(char, letterSpacing, cache, context, bitmapFont, scale);
        };
        const ellipsisWidth = Array.from(ellipsis).reduce((prev, cur) => {
            return prev + calcWidth(cur);
        }, 0);
        function appendEllipsis(lineIndex) {
            // If there is not enough space to display the string itself, it is clipped.
            // @see https://developer.mozilla.org/en-US/docs/Web/CSS/text-overflow#values
            if (ellipsisWidth <= 0 || ellipsisWidth > maxWidth) {
                return;
            }
            // Backspace from line's end.
            const currentLineLength = lines[lineIndex].length;
            let lastLineWidth = 0;
            let lastLineIndex = currentLineLength;
            for (let i = 0; i < currentLineLength; i++) {
                const width = calcWidth(lines[lineIndex][i]);
                if (lastLineWidth + width + ellipsisWidth > maxWidth) {
                    lastLineIndex = i;
                    break;
                }
                lastLineWidth += width;
            }
            lines[lineIndex] =
                (lines[lineIndex] || '').slice(0, lastLineIndex) + ellipsis;
        }
        const chars = Array.from(text);
        for (let i = 0; i < chars.length; i++) {
            const char = chars[i];
            const prevChar = text[i - 1];
            const nextChar = text[i + 1];
            const charWidth = calcWidth(char);
            if (this.isNewline(char)) {
                currentIndex++;
                // exceed maxLines, break immediately
                if (currentIndex >= maxLines) {
                    // parsedStyle.isOverflowing = true;
                    if (i < chars.length - 1) {
                        appendEllipsis(currentIndex - 1);
                    }
                    break;
                }
                currentWidth = 0;
                lines[currentIndex] = '';
                continue;
            }
            if (currentWidth > 0 && currentWidth + charWidth > maxWidth) {
                if (currentIndex + 1 >= maxLines) {
                    // parsedStyle.isOverflowing = true;
                    appendEllipsis(currentIndex);
                    break;
                }
                currentIndex++;
                currentWidth = 0;
                lines[currentIndex] = '';
                if (this.isBreakingSpace(char)) {
                    continue;
                }
                if (!this.canBreakInLastChar(char)) {
                    lines = this.trimToBreakable(lines);
                    currentWidth = this.sumTextWidthByCache(lines[currentIndex] || '', cache);
                }
                if (this.shouldBreakByKinsokuShorui(char, nextChar)) {
                    lines = this.trimByKinsokuShorui(lines);
                    currentWidth += calcWidth(prevChar || '');
                }
            }
            currentWidth += charWidth;
            lines[currentIndex] = (lines[currentIndex] || '') + char;
        }
        return lines.join('\n');
    }
    computeBidi(text) {
        if (!__classPrivateFieldGet(this, _ComputeTextMetrics_bidiCache, "f")[text]) {
            // @see https://github.com/beanandbean/font-mesh-pipeline/blob/main/packages/harfbuzz-modern-wrapper/src/harfbuzz.ts#L50
            const segmentStack = [new Array()];
            const reduceStack = (stack, target) => {
                const validatedTarget = target < 0 ? 0 : target;
                while (stack.length > validatedTarget + 1) {
                    const current = stack.pop();
                    stack[stack.length - 1].push(...current.reverse());
                }
            };
            const pushInStack = (stack, text, level) => {
                if (level + 1 > stack.length) {
                    stack.push(...Array.from({ length: level + 1 - stack.length }, () => new Array()));
                }
                else {
                    reduceStack(stack, level);
                }
                stack[level].push({
                    text,
                    direction: level % 2 === 0 ? 'ltr' : 'rtl',
                });
            };
            const embeddingLevels = __classPrivateFieldGet(this, _ComputeTextMetrics_bidi, "f").getEmbeddingLevels(text);
            const iter = embeddingLevels.levels.entries();
            const first = iter.next();
            if (!first.done) {
                let [prevIndex, prevLevel] = first.value;
                for (const [i, level] of iter) {
                    if (level !== prevLevel) {
                        pushInStack(segmentStack, text.slice(prevIndex, i), prevLevel);
                        prevIndex = i;
                        prevLevel = level;
                    }
                }
                pushInStack(segmentStack, text.slice(prevIndex), prevLevel);
                reduceStack(segmentStack, 0);
            }
            let bidiChars = '';
            for (const segment of segmentStack[0]) {
                const { text, direction } = segment;
                bidiChars +=
                    direction === 'ltr' ? text : text.split('').reverse().join('');
            }
            __classPrivateFieldGet(this, _ComputeTextMetrics_bidiCache, "f")[text] = bidiChars;
        }
        return __classPrivateFieldGet(this, _ComputeTextMetrics_bidiCache, "f")[text];
    }
    measureBitmapFont(bitmapFont, fontSize) {
        const { fontMetrics, lineHeight } = bitmapFont;
        const scale = fontSize / bitmapFont.baseMeasurementFontSize;
        return {
            scale,
            lineHeight: lineHeight * scale,
            fontMetrics: {
                actualBoundingBoxAscent: fontMetrics.ascent * scale,
                actualBoundingBoxDescent: fontMetrics.descent * scale,
                fontSize,
            },
        };
    }
    measureTextInternal(text, letterSpacing, bitmapFont, bitmapFontKerning, scale) {
        const segments = environment_1.DOMAdapter.get().splitGraphemes(text);
        let metricWidth;
        let boundsWidth;
        let previousChar;
        if (bitmapFont) {
            metricWidth = segments.reduce((sum, char) => {
                var _a, _b;
                const advance = (_a = bitmapFont.chars[char]) === null || _a === void 0 ? void 0 : _a.xAdvance;
                const kerning = (bitmapFontKerning &&
                    previousChar &&
                    ((_b = bitmapFont.chars[char]) === null || _b === void 0 ? void 0 : _b.kerning[previousChar])) ||
                    0;
                previousChar = char;
                return sum + ((advance + kerning) * scale || 0);
            }, 0);
            boundsWidth = metricWidth;
        }
        else {
            __classPrivateFieldGet(this, _ComputeTextMetrics_context, "f").letterSpacing = `${letterSpacing}px`;
            const metrics = __classPrivateFieldGet(this, _ComputeTextMetrics_context, "f").measureText(text);
            metricWidth = metrics.width;
            const actualBoundingBoxLeft = -metrics.actualBoundingBoxLeft;
            const actualBoundingBoxRight = metrics.actualBoundingBoxRight;
            boundsWidth = actualBoundingBoxRight - actualBoundingBoxLeft;
        }
        if (metricWidth > 0) {
            const val = (segments.length - 1) * letterSpacing;
            metricWidth += val;
            boundsWidth += val;
        }
        return Math.max(metricWidth, boundsWidth);
    }
    isBreakingSpace(char) {
        return BREAKING_SPACES.includes(char.charCodeAt(0));
    }
    isNewline(char) {
        return NEWLINES.includes(char.charCodeAt(0));
    }
    trimToBreakable(prev) {
        const next = [...prev];
        const prevLine = next[next.length - 2];
        const index = this.findBreakableIndex(prevLine);
        if (index === -1 || !prevLine)
            return next;
        const trimmedChar = prevLine.slice(index, index + 1);
        const isTrimmedWithSpace = this.isBreakingSpace(trimmedChar);
        const trimFrom = index + 1;
        const trimTo = index + (isTrimmedWithSpace ? 0 : 1);
        next[next.length - 1] += prevLine.slice(trimFrom, prevLine.length);
        next[next.length - 2] = prevLine.slice(0, trimTo);
        return next;
    }
    canBreakInLastChar(char) {
        if (char && LATIN_REGEX.test(char))
            return false;
        return true;
    }
    sumTextWidthByCache(text, cache) {
        return text.split('').reduce((sum, c) => {
            if (!cache[c])
                throw Error('cannot count the word without cache');
            return sum + cache[c];
        }, 0);
    }
    findBreakableIndex(line) {
        for (let i = line.length - 1; i >= 0; i--) {
            if (!LATIN_REGEX.test(line[i]))
                return i;
        }
        return -1;
    }
    getFromCache(key, letterSpacing, cache, context, bitmapFont, scale) {
        var _a;
        let width = cache[key];
        if (typeof width !== 'number') {
            const spacing = key.length * letterSpacing;
            width =
                (bitmapFont
                    ? ((_a = bitmapFont.chars[key]) === null || _a === void 0 ? void 0 : _a.xAdvance) || 0
                    : context.measureText(key).width) *
                    scale +
                    spacing;
            cache[key] = width;
        }
        return width;
    }
    collapseSpaces(whiteSpace) {
        return whiteSpace === 'normal' || whiteSpace === 'pre-line';
    }
    collapseNewlines(whiteSpace) {
        return whiteSpace === 'normal';
    }
}
exports.ComputeTextMetrics = ComputeTextMetrics;
_ComputeTextMetrics_canvas = new WeakMap(), _ComputeTextMetrics_context = new WeakMap(), _ComputeTextMetrics_bidi = new WeakMap(), _ComputeTextMetrics_bidiCache = new WeakMap(), _ComputeTextMetrics_fonts = new WeakMap();
//# sourceMappingURL=ComputeTextMetrics.js.map