"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExportSVG = void 0;
const becsy_1 = require("@lastolivegames/becsy");
const components_1 = require("../components");
const environment_1 = require("../environment");
const utils_1 = require("../utils");
class ExportSVG extends becsy_1.System {
    *setScreenshotTrigger(canvas, dataURL) {
        if (!canvas.has(components_1.Screenshot)) {
            canvas.add(components_1.Screenshot);
        }
        const screenshot = canvas.write(components_1.Screenshot);
        Object.assign(screenshot, { dataURL });
        yield;
        Object.assign(screenshot, { dataURL: '' });
    }
    constructor() {
        super();
        this.canvases = this.query((q) => q.current.with(components_1.Canvas).read);
        this.query((q) => q
            .using(components_1.Theme, components_1.Grid, components_1.Camera, components_1.Name, components_1.Parent, components_1.Children, components_1.Circle, components_1.Ellipse, components_1.Rect, components_1.Polyline, components_1.Path, components_1.Text, components_1.Stroke, components_1.Opacity, components_1.Transform, components_1.FillSolid, components_1.FillGradient, components_1.FillPattern, components_1.FillImage, components_1.DropShadow, components_1.InnerShadow, components_1.Visibility)
            .read.and.using(components_1.VectorScreenshotRequest, components_1.Screenshot).write);
    }
    execute() {
        this.canvases.current.forEach((canvas) => {
            if (!canvas.has(components_1.VectorScreenshotRequest)) {
                return;
            }
            const vectorScreenshotRequest = canvas.read(components_1.VectorScreenshotRequest);
            if (!vectorScreenshotRequest.enabled) {
                return;
            }
            this.setScreenshotTrigger(canvas, this.toSVGDataURL(canvas, vectorScreenshotRequest));
        });
    }
    toSVG(canvas, options = {}) {
        const { grid: gridEnabled } = options;
        const { width, height, cameras } = canvas.read(components_1.Canvas);
        const { mode, colors } = canvas.read(components_1.Theme);
        const { checkboardStyle } = canvas.read(components_1.Grid);
        const { grid: gridColor, background: backgroundColor } = colors[mode];
        const $namespace = (0, utils_1.createSVGElement)('svg');
        $namespace.setAttribute('width', `${width}`);
        $namespace.setAttribute('height', `${height}`);
        // @see https://www.geeksforgeeks.org/how-to-set-the-svg-background-color/
        $namespace.setAttribute('style', `background-color: ${backgroundColor}`);
        if (gridEnabled) {
            if (checkboardStyle === components_1.CheckboardStyle.GRID) {
                this.drawLinesGrid($namespace, gridColor);
            }
            else if (checkboardStyle === components_1.CheckboardStyle.DOTS) {
                this.drawDotsGrid($namespace, gridColor);
            }
        }
        // console.log(entityToSerializedNodes(entity));
        (0, utils_1.serializeNodesToSVGElements)((0, utils_1.entityToSerializedNodes)(cameras[0])).forEach((element) => {
            $namespace.appendChild(element);
        });
        return $namespace;
    }
    toSVGDataURL(entity, options = {}) {
        const $namespace = this.toSVG(entity, options);
        const svgDocType = environment_1.DOMAdapter.get()
            .getDocument()
            .implementation.createDocumentType('svg', '-//W3C//DTD SVG 1.1//EN', 'http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd');
        const svgDoc = environment_1.DOMAdapter.get()
            .getDocument()
            .implementation.createDocument('http://www.w3.org/2000/svg', 'svg', svgDocType);
        svgDoc.replaceChild($namespace, svgDoc.documentElement);
        return `data:image/svg+xml;charset=utf8,${encodeURIComponent(environment_1.DOMAdapter.get().getXMLSerializer().serializeToString(svgDoc))}`;
    }
    drawLinesGrid($namespace, gridColor) {
        const $defs = (0, utils_1.createSVGElement)('defs');
        $namespace.appendChild($defs);
        const $pattern = (0, utils_1.createSVGElement)('pattern');
        $pattern.setAttribute('id', 'small-grid');
        $pattern.setAttribute('width', '10');
        $pattern.setAttribute('height', '10');
        $pattern.setAttribute('patternUnits', 'userSpaceOnUse');
        const $path = (0, utils_1.createSVGElement)('path');
        $path.setAttribute('d', 'M 10 0 L 0 0 0 10');
        $path.setAttribute('fill', 'none');
        $path.setAttribute('stroke', gridColor);
        $path.setAttribute('stroke-width', '1');
        $pattern.appendChild($path);
        const $pattern2 = (0, utils_1.createSVGElement)('pattern');
        $pattern2.setAttribute('id', 'grid');
        $pattern2.setAttribute('width', '100');
        $pattern2.setAttribute('height', '100');
        $pattern2.setAttribute('patternUnits', 'userSpaceOnUse');
        const $rect = (0, utils_1.createSVGElement)('rect');
        $rect.setAttribute('width', '100');
        $rect.setAttribute('height', '100');
        $rect.setAttribute('fill', 'url(#small-grid)');
        $pattern2.appendChild($rect);
        const $path2 = (0, utils_1.createSVGElement)('path');
        $path2.setAttribute('d', 'M 100 0 L 0 0 0 100');
        $path2.setAttribute('fill', 'none');
        $path2.setAttribute('stroke', gridColor);
        $path2.setAttribute('stroke-width', '2');
        $pattern2.appendChild($path2);
        $defs.appendChild($pattern);
        $defs.appendChild($pattern2);
        const $rect2 = (0, utils_1.createSVGElement)('rect');
        $rect2.setAttribute('width', '100%');
        $rect2.setAttribute('height', '100%');
        $rect2.setAttribute('fill', 'url(#grid)');
        $namespace.appendChild($rect2);
    }
    drawDotsGrid($namespace, gridColor) {
        const $defs = (0, utils_1.createSVGElement)('defs');
        $namespace.appendChild($defs);
        const $circleTL = (0, utils_1.createSVGElement)('circle');
        $circleTL.setAttribute('id', 'dot-tl');
        $circleTL.setAttribute('cx', '0');
        $circleTL.setAttribute('cy', '0');
        $circleTL.setAttribute('r', '2');
        $circleTL.setAttribute('fill', gridColor);
        const $circleTR = $circleTL.cloneNode();
        const $circleBL = $circleTL.cloneNode();
        const $circleBR = $circleTL.cloneNode();
        $circleTR.setAttribute('id', 'dot-tr');
        $circleTR.setAttribute('cx', '20');
        $circleBL.setAttribute('id', 'dot-bl');
        $circleBL.setAttribute('cy', '20');
        $circleBR.setAttribute('id', 'dot-br');
        $circleBR.setAttribute('cx', '20');
        $circleBR.setAttribute('cy', '20');
        $defs.appendChild($circleTL);
        $defs.appendChild($circleTR);
        $defs.appendChild($circleBL);
        $defs.appendChild($circleBR);
        const $pattern = (0, utils_1.createSVGElement)('pattern');
        $pattern.setAttribute('id', 'dots-grid');
        $pattern.setAttribute('width', '20');
        $pattern.setAttribute('height', '20');
        $pattern.setAttribute('patternUnits', 'userSpaceOnUse');
        const $useBL = (0, utils_1.createSVGElement)('use');
        $useBL.setAttribute('xlink:href', '#dot-bl');
        const $useBR = (0, utils_1.createSVGElement)('use');
        $useBR.setAttribute('xlink:href', '#dot-br');
        const $useTL = (0, utils_1.createSVGElement)('use');
        $useTL.setAttribute('xlink:href', '#dot-tl');
        const $useTR = (0, utils_1.createSVGElement)('use');
        $useTR.setAttribute('xlink:href', '#dot-tr');
        $pattern.appendChild($useBL);
        $pattern.appendChild($useBR);
        $pattern.appendChild($useTL);
        $pattern.appendChild($useTR);
        $defs.appendChild($pattern);
        const $rect = (0, utils_1.createSVGElement)('rect');
        $rect.setAttribute('width', '100%');
        $rect.setAttribute('height', '100%');
        $rect.setAttribute('fill', 'url(#dots-grid)');
        $namespace.appendChild($rect);
    }
}
exports.ExportSVG = ExportSVG;
__decorate([
    becsy_1.co
], ExportSVG.prototype, "setScreenshotTrigger", null);
//# sourceMappingURL=ExportSVG.js.map