"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, state, value, kind, f) {
    if (kind === "m") throw new TypeError("Private method is not writable");
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a setter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot write private member to an object whose class did not declare it");
    return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _SetupDevice_texturePool;
Object.defineProperty(exports, "__esModule", { value: true });
exports.SetupDevice = void 0;
const becsy_1 = require("@lastolivegames/becsy");
const g_device_api_1 = require("@antv/g-device-api");
const components_1 = require("../components");
const utils_1 = require("../utils");
const resources_1 = require("../resources");
/**
 * Usually the first built-in system to run.
 * It will create a new device and swap chain for each canvas.
 */
class SetupDevice extends becsy_1.System {
    constructor() {
        super();
        this.canvases = this.query((q) => q.added.and.changed.and.removed.with(components_1.Canvas).trackWrites);
        _SetupDevice_texturePool.set(this, void 0);
        this.query((q) => q.using(components_1.GPUResource, components_1.Canvas, components_1.Theme, components_1.Grid).write);
        __classPrivateFieldSet(this, _SetupDevice_texturePool, new resources_1.TexturePool(), "f");
    }
    *addGPUResource(canvas, gpuResource) {
        canvas.add(components_1.GPUResource, gpuResource);
        yield;
    }
    execute() {
        this.canvases.added.forEach((canvas) => __awaiter(this, void 0, void 0, function* () {
            if (!canvas.has(components_1.Theme)) {
                canvas.add(components_1.Theme);
            }
            if (!canvas.has(components_1.Grid)) {
                canvas.add(components_1.Grid);
            }
            const { width, height, devicePixelRatio, renderer, shaderCompilerPath, element, } = canvas.read(components_1.Canvas);
            const holder = canvas.hold();
            const widthDPR = width * devicePixelRatio;
            const heightDPR = height * devicePixelRatio;
            let deviceContribution;
            if (renderer === 'webgl') {
                deviceContribution = new g_device_api_1.WebGLDeviceContribution({
                    targets: ['webgl1'],
                    antialias: true,
                    shaderDebug: true,
                    trackResources: false,
                    onContextCreationError: () => { },
                    onContextLost: () => { },
                    onContextRestored(e) { },
                });
            }
            else {
                deviceContribution = new g_device_api_1.WebGPUDeviceContribution({
                    shaderCompilerPath,
                    onContextLost: () => { },
                });
            }
            const swapChain = yield deviceContribution.createSwapChain(element);
            swapChain.configureSwapChain(widthDPR, heightDPR);
            const device = swapChain.getDevice();
            const [renderTarget, depthRenderTarget] = this.createRenderTarget(device, widthDPR, heightDPR);
            const renderCache = new utils_1.RenderCache(device);
            this.addGPUResource(holder, {
                device,
                swapChain,
                renderTarget,
                depthRenderTarget,
                renderCache,
                texturePool: __classPrivateFieldGet(this, _SetupDevice_texturePool, "f"),
            });
        }));
        this.canvases.changed.forEach((canvas) => {
            if (!canvas.has(components_1.GPUResource)) {
                return;
            }
            const { width, height, devicePixelRatio } = canvas.read(components_1.Canvas);
            const widthDPR = width * devicePixelRatio;
            const heightDPR = height * devicePixelRatio;
            const { swapChain, device, renderTarget: oldRenderTarget, depthRenderTarget: oldDepthRenderTarget, } = canvas.read(components_1.GPUResource);
            swapChain.configureSwapChain(widthDPR, heightDPR);
            oldRenderTarget.destroy();
            oldDepthRenderTarget.destroy();
            const [renderTarget, depthRenderTarget] = this.createRenderTarget(device, widthDPR, heightDPR);
            Object.assign(canvas.write(components_1.GPUResource), {
                renderTarget,
                depthRenderTarget,
            });
        });
        this.canvases.removed.forEach((canvas) => {
            this.accessRecentlyDeletedData();
            this.destroyCanvas(canvas);
        });
    }
    finalize() {
        this.canvases.current.forEach((canvas) => {
            this.destroyCanvas(canvas);
        });
        __classPrivateFieldGet(this, _SetupDevice_texturePool, "f").destroy();
    }
    createRenderTarget(device, width, height) {
        const renderTarget = device.createRenderTargetFromTexture(device.createTexture({
            format: g_device_api_1.Format.U8_RGBA_RT,
            width,
            height,
            usage: g_device_api_1.TextureUsage.RENDER_TARGET,
        }));
        const depthRenderTarget = device.createRenderTargetFromTexture(device.createTexture({
            format: g_device_api_1.Format.D24_S8,
            width,
            height,
            usage: g_device_api_1.TextureUsage.RENDER_TARGET,
        }));
        return [renderTarget, depthRenderTarget];
    }
    destroyCanvas(canvas) {
        const { device, renderTarget, depthRenderTarget, renderCache } = canvas.read(components_1.GPUResource);
        renderCache.destroy();
        renderTarget.destroy();
        depthRenderTarget.destroy();
        device.destroy();
        device.checkForLeaks();
    }
}
exports.SetupDevice = SetupDevice;
_SetupDevice_texturePool = new WeakMap();
__decorate([
    becsy_1.co
], SetupDevice.prototype, "addGPUResource", null);
//# sourceMappingURL=SetupDevice.js.map