"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractBitmapFont = void 0;
const eventemitter3_1 = __importDefault(require("eventemitter3"));
/**
 * An abstract representation of a bitmap font.
 * @memberof text
 */
class AbstractBitmapFont extends eventemitter3_1.default {
    constructor() {
        super(...arguments);
        /** The map of characters by character code. */
        this.chars = Object.create(null);
        /**
         * The line-height of the font face in pixels.
         * @type {number}
         */
        this.lineHeight = 0;
        /**
         * The name of the font face
         * @type {string}
         */
        this.fontFamily = '';
        /** The metrics of the font face. */
        this.fontMetrics = {
            fontSize: 0,
            ascent: 0,
            descent: 0,
        };
        /**
         * The offset of the font face from the baseline.
         * @type {number}
         */
        this.baseLineOffset = 0;
        /** The range and type of the distance field for this font. */
        this.distanceField = {
            type: 'none',
            range: 0,
        };
        /** The map of base page textures (i.e., sheets of glyphs). */
        this.pages = [];
        /** should the fill for this font be applied as a tint to the text. */
        this.applyFillAsTint = true;
        /** The size of the font face in pixels. */
        this.baseMeasurementFontSize = 100;
        this.baseRenderedFontSize = 100;
    }
    destroy(destroyTextures = false) {
        var _a;
        this.emit('destroy', this);
        this.removeAllListeners();
        for (const i in this.chars) {
            // texture may not exist if the char is " ", \n, \r, or \t.
            (_a = this.chars[i].texture) === null || _a === void 0 ? void 0 : _a.destroy();
        }
        this.chars = null;
        if (destroyTextures) {
            this.pages.forEach((page) => page.texture.destroy());
            this.pages = null;
        }
    }
}
exports.AbstractBitmapFont = AbstractBitmapFont;
//# sourceMappingURL=AbstractBitmapFont.js.map