"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BitmapFont = void 0;
const g_device_api_1 = require("@antv/g-device-api");
const AbstractBitmapFont_1 = require("./AbstractBitmapFont");
class BitmapFont extends AbstractBitmapFont_1.AbstractBitmapFont {
    constructor(options) {
        var _a;
        super();
        this.options = options;
        const { data } = this.options;
        Object.keys(data.chars).forEach((key) => {
            var _a;
            const charData = data.chars[key];
            const rect = {
                // charData.x + textureFrame.x,
                // charData.y + textureFrame.y,
                x: charData.x,
                y: charData.y,
                w: charData.width,
                h: charData.height,
            };
            this.chars[key] = {
                id: key.codePointAt(0),
                xOffset: charData.xOffset,
                yOffset: charData.yOffset,
                xAdvance: charData.xAdvance,
                kerning: (_a = charData.kerning) !== null && _a !== void 0 ? _a : {},
                rect,
            };
        });
        this.baseRenderedFontSize = data.fontSize;
        this.baseMeasurementFontSize = data.fontSize;
        this.fontMetrics = {
            ascent: 0,
            descent: 0,
            fontSize: data.fontSize,
        };
        this.baseLineOffset = data.baseLineOffset;
        this.lineHeight = data.lineHeight;
        this.fontFamily = data.fontFamily;
        this.distanceField =
            (_a = data.distanceField) !== null && _a !== void 0 ? _a : {
                type: 'none',
                range: 0,
            };
    }
    createTexture(device) {
        const { images, data } = this.options;
        const textures = images.map((image) => {
            const texture = device.createTexture({
                format: g_device_api_1.Format.U8_RGBA_NORM,
                width: image.width,
                height: image.height,
                usage: g_device_api_1.TextureUsage.SAMPLED,
            });
            texture.setImageData([image]);
            return {
                texture,
                width: image.width,
                height: image.height,
            };
        });
        Object.keys(data.pages).forEach((key) => {
            const pageData = data.pages[parseInt(key, 10)];
            const { texture, width, height } = textures[pageData.id];
            this.pages.push({ texture, width, height });
        });
        Object.keys(data.chars).forEach((key) => {
            const charData = data.chars[key];
            const { texture } = textures[charData.page];
            this.chars[key].texture = texture;
        });
    }
    destroy() {
        super.destroy();
        for (let i = 0; i < this.pages.length; i++) {
            const { texture } = this.pages[i];
            texture.destroy();
        }
        this.pages = null;
    }
}
exports.BitmapFont = BitmapFont;
//# sourceMappingURL=BitmapFont.js.map