"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.bitmapFontJSONParser = void 0;
exports.bitmapFontJSONParser = {
    test(data) {
        try {
            JSON.parse(data);
            return true;
        }
        catch (e) {
            return false;
        }
    },
    parse(jsonStr) {
        var _a, _b, _c, _d;
        const data = {
            chars: {},
            pages: [],
            lineHeight: 0,
            fontSize: 0,
            fontFamily: '',
            distanceField: null,
            baseLineOffset: 0,
        };
        const json = JSON.parse(jsonStr);
        const info = json.info;
        const common = json.common;
        const distanceField = json.distanceField;
        if (distanceField) {
            data.distanceField = {
                type: distanceField.fieldType,
                range: distanceField.distanceRange,
            };
        }
        // pages and chars:
        const pages = json.pages;
        const chars = json.chars;
        const kernings = json.kernings;
        data.fontSize = info.size;
        data.fontFamily = info.face;
        data.lineHeight = common.lineHeight;
        for (let i = 0; i < pages.length; i++) {
            data.pages.push({
                id: ((_a = pages[i]) === null || _a === void 0 ? void 0 : _a.id) || 0,
                file: ((_b = pages[i]) === null || _b === void 0 ? void 0 : _b.file) || pages[i],
            });
        }
        const map = {};
        data.baseLineOffset = data.lineHeight - common.base;
        for (let i = 0; i < chars.length; i++) {
            const charNode = chars[i];
            const id = charNode.id;
            let letter = (_d = (_c = charNode.letter) !== null && _c !== void 0 ? _c : charNode.char) !== null && _d !== void 0 ? _d : String.fromCharCode(id);
            if (letter === 'space')
                letter = ' ';
            map[id] = letter;
            data.chars[letter] = {
                id,
                // texture deets..
                page: charNode.page || 0,
                x: charNode.x,
                y: charNode.y,
                width: charNode.width,
                height: charNode.height,
                // render deets..
                xOffset: charNode.xoffset,
                yOffset: charNode.yoffset, // + baseLineOffset,
                xAdvance: charNode.xadvance,
                kerning: {},
            };
        }
        for (let i = 0; i < kernings.length; i++) {
            const kerningNode = kernings[i];
            const first = kerningNode.first;
            const second = kerningNode.second;
            const amount = kerningNode.amount;
            data.chars[map[second]].kerning[map[first]] = amount; // * 10000;
        }
        return data;
    },
};
//# sourceMappingURL=bitmap-font-json-parser.js.map