"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadBitmapFont = void 0;
const core_1 = require("@loaders.gl/core");
const images_1 = require("@loaders.gl/images");
const BitmapFont_1 = require("./BitmapFont");
const bitmap_font_text_parser_1 = require("./bitmap-font-text-parser");
const bitmap_font_xml_string_parser_1 = require("./bitmap-font-xml-string-parser");
const bitmap_font_json_parser_1 = require("./bitmap-font-json-parser");
const validExtensions = ['.xml', '.fnt'];
exports.loadBitmapFont = {
    test(url) {
        return validExtensions.includes(url.split('.').pop().toLowerCase());
    },
    parse(asset) {
        return __awaiter(this, void 0, void 0, function* () {
            const bitmapFontData = bitmap_font_json_parser_1.bitmapFontJSONParser.test(asset)
                ? bitmap_font_json_parser_1.bitmapFontJSONParser.parse(asset)
                : bitmap_font_text_parser_1.bitmapFontTextParser.test(asset)
                    ? bitmap_font_text_parser_1.bitmapFontTextParser.parse(asset)
                    : bitmap_font_xml_string_parser_1.bitmapFontXMLStringParser.parse(asset);
            const { pages } = bitmapFontData;
            const textureUrls = [];
            // if we have a distance field - we can assume this is a signed distance field font
            // and we should use force linear filtering and no alpha premultiply
            const textureOptions = bitmapFontData.distanceField
                ? {
                    scaleMode: 'linear',
                    alphaMode: 'premultiply-alpha-on-upload',
                    autoGenerateMipmaps: false,
                    resolution: 1,
                }
                : {};
            for (let i = 0; i < pages.length; ++i) {
                const pageFile = pages[i].file;
                const imagePath = pageFile;
                // imagePath = copySearchParams(imagePath, src);
                textureUrls.push({
                    src: imagePath,
                    data: textureOptions,
                });
            }
            const images = yield Promise.all(textureUrls.map((_a) => __awaiter(this, [_a], void 0, function* ({ src }) { return (yield (0, core_1.load)(src, images_1.ImageLoader)); })));
            return new BitmapFont_1.BitmapFont({
                data: bitmapFontData,
                images,
            });
        });
    },
};
//# sourceMappingURL=load-bitmap-font.js.map