"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.serializedNodesToEntities = serializedNodesToEntities;
const util_1 = require("@antv/util");
const toposort_1 = __importDefault(require("toposort"));
const __1 = require("../..");
const points_1 = require("./points");
function serializedNodesToEntities(nodes, commands) {
    var _a;
    const vertices = nodes.map((node) => node.id);
    const edges = nodes
        .filter((node) => !(0, util_1.isNil)(node.parentId))
        .map((node) => [node.parentId, node.id]);
    const sorted = toposort_1.default.array(vertices, edges);
    const idEntityMap = new Map();
    const entities = [];
    for (const id of sorted) {
        const node = nodes.find((node) => node.id === id);
        const { parentId, type } = node;
        const attributes = node;
        const entity = commands.spawn();
        idEntityMap.set(id, entity);
        const { transform } = attributes;
        entity.insert(new __1.Transform(transform));
        if (type !== 'g') {
            entity.insert(new __1.Renderable());
        }
        if (type === 'circle') {
            const { cx, cy, r } = attributes;
            entity.insert(new __1.Circle({ cx, cy, r }));
        }
        else if (type === 'ellipse') {
            const { cx, cy, rx, ry } = attributes;
            entity.insert(new __1.Ellipse({ cx, cy, rx, ry }));
        }
        else if (type === 'rect') {
            const { x, y, width, height, cornerRadius } = attributes;
            entity.insert(new __1.Rect({ x, y, width, height, cornerRadius }));
        }
        else if (type === 'polyline') {
            const { points } = attributes;
            entity.insert(new __1.Polyline({ points: (0, points_1.deserializePoints)(points) }));
        }
        else if (type === 'path') {
            const { d, fillRule, tessellationMethod } = attributes;
            entity.insert(new __1.Path({ d, fillRule, tessellationMethod }));
        }
        else if (type === 'text') {
            const { x, y, content, fontFamily, fontSize, fontWeight, fontStyle, fontVariant, letterSpacing, lineHeight, whiteSpace, wordWrap, wordWrapWidth, textAlign, textBaseline, } = attributes;
            entity.insert(new __1.Text({
                x,
                y,
                content,
                fontFamily,
                fontSize,
                fontWeight,
                fontStyle,
                fontVariant,
                letterSpacing,
                lineHeight,
                whiteSpace,
                wordWrap,
                wordWrapWidth,
                textAlign,
                textBaseline,
            }));
        }
        const { fill, fillOpacity, opacity } = attributes;
        if (fill) {
            entity.insert(new __1.FillSolid(fill));
            // TODO: gradient, pattern, etc.
        }
        const { stroke, strokeWidth, strokeDasharray, strokeLinecap, strokeLinejoin, strokeMiterlimit, strokeOpacity, strokeDashoffset, strokeAlignment, } = attributes;
        if (stroke) {
            entity.insert(new __1.Stroke({
                color: stroke,
                width: strokeWidth,
                dasharray: strokeDasharray === null || strokeDasharray === void 0 ? void 0 : strokeDasharray.split(',').map(Number),
                linecap: strokeLinecap,
                linejoin: strokeLinejoin,
                miterlimit: strokeMiterlimit,
                dashoffset: strokeDashoffset,
                alignment: strokeAlignment,
            }));
        }
        if (opacity || fillOpacity || strokeOpacity) {
            entity.insert(new __1.Opacity({
                opacity,
                fillOpacity,
                strokeOpacity,
            }));
        }
        const { visibility } = attributes;
        entity.insert(new __1.Visibility(visibility));
        const { name } = attributes;
        entity.insert(new __1.Name(name));
        if (parentId) {
            (_a = idEntityMap.get(parentId)) === null || _a === void 0 ? void 0 : _a.appendChild(entity);
        }
        entities.push(entity.id().hold());
    }
    return { entities, idEntityMap };
}
//# sourceMappingURL=entity.js.map