"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GlyphAtlas = exports.glyphPadding = void 0;
const potpack_1 = __importDefault(require("potpack"));
const alpha_image_1 = require("./alpha-image");
const glyph_manager_1 = require("./glyph-manager");
exports.glyphPadding = 1;
/**
 * The glyph padding is just to prevent sampling errors at the boundaries between
 * glyphs in the atlas texture, and for that purpose there's no need to make it
 * bigger with high-res SDFs. However, layout is done based on the glyph size
 * including this padding, so scaling this padding is the easiest way to keep
 * layout exactly the same as the SDF_SCALE changes.
 */
const padding = exports.glyphPadding * glyph_manager_1.SDF_SCALE;
/**
 * Merge SDFs into a large squared atlas with `potpack`,
 * because on WebGL1 context, all textures are resized to a power of two to produce the best quality.
 *
 * @see https://doc.babylonjs.com/advanced_topics/webGL2#power-of-two-textures
 */
class GlyphAtlas {
    constructor(stacks) {
        const positions = {};
        const bins = [];
        for (const stack in stacks) {
            const glyphs = stacks[stack];
            const stackPositions = (positions[stack] = {});
            for (const id in glyphs) {
                const src = glyphs[id];
                if (!src || src.bitmap.width === 0 || src.bitmap.height === 0)
                    continue;
                const bin = {
                    x: 0,
                    y: 0,
                    w: src.bitmap.width + 2 * padding,
                    h: src.bitmap.height + 2 * padding,
                };
                bins.push(bin);
                stackPositions[id] = { rect: bin, metrics: src.metrics };
            }
        }
        const { w, h } = (0, potpack_1.default)(bins);
        const image = new alpha_image_1.RGBAImage({ width: w || 1, height: h || 1 });
        for (const stack in stacks) {
            const glyphs = stacks[stack];
            for (const id in glyphs) {
                const src = glyphs[id];
                if (!src || src.bitmap.width === 0 || src.bitmap.height === 0)
                    continue;
                const bin = positions[stack][id].rect;
                alpha_image_1.RGBAImage.copy(src.bitmap, image, { x: 0, y: 0 }, { x: bin.x + padding, y: bin.y + padding }, src.bitmap);
            }
        }
        this.image = image;
        this.positions = positions;
    }
}
exports.GlyphAtlas = GlyphAtlas;
//# sourceMappingURL=glyph-atlas.js.map