/**
 * @see https://github.com/mapbox/mapbox-gl-js/blob/main/src/render/glyph_manager.ts
 */
import type { Device, Texture } from '@antv/g-device-api';
import type { StyleGlyph } from './alpha-image';
import { GlyphAtlas } from './glyph-atlas';
import { BitmapFont } from '../bitmap-font/BitmapFont';
export type PositionedGlyph = {
    glyph: string;
    x: number;
    y: number;
    scale: number;
    fontStack: string;
};
/**
 * SDF_SCALE controls the pixel density of locally generated glyphs relative
 * to "normal" SDFs which are generated at 24pt font and a "pixel ratio" of 1.
 * The GlyphManager will generate glyphs SDF_SCALE times as large,
 * but with the same glyph metrics, and the quad generation code will scale them
 * back down so they display at the same size.
 *
 * The choice of SDF_SCALE is a trade-off between performance and quality.
 * Glyph generation time grows quadratically with the the scale, while quality
 * improvements drop off rapidly when the scale is higher than the pixel ratio
 * of the device. The scale of 2 buys noticeable improvements on HDPI screens
 * at acceptable cost.
 */
export declare const SDF_SCALE = 2;
export declare const BASE_FONT_WIDTH: number;
export declare const BASE_FONT_BUFFER: number;
export declare const RADIUS: number;
export declare function getDefaultCharacterSet(): string[];
export declare class GlyphManager {
    private sdfGeneratorCache;
    private glyphAtlas;
    private glyphMap;
    private glyphAtlasTexture;
    constructor();
    destroy(): void;
    getMap(): Record<string, Record<string, StyleGlyph>>;
    getAtlas(): GlyphAtlas;
    getAtlasTexture(): Texture;
    layout(lines: string[], fontStack: string, lineHeight: number, textAlign: CanvasTextAlign, letterSpacing: number, bitmapFont?: BitmapFont, scale?: number, bitmapFontKerning?: boolean, dx?: number, dy?: number): PositionedGlyph[];
    generateAtlas(fontStack: string, fontFamily: string, fontWeight: string, fontStyle: string, text: string, device: Device, esdt: boolean, fill: string): void;
    private generateSDF;
}
