"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.glyphToEDT = void 0;
const tiny_sdf_1 = require("./tiny-sdf");
// Paint glyph data into stage
const paintIntoStage = (stage, data, w, h, pad) => {
    const wp = w + pad * 2;
    const hp = h + pad * 2;
    const np = wp * hp;
    const { outer, inner } = stage;
    outer.fill(tiny_sdf_1.INF, 0, np);
    inner.fill(0, 0, np);
    const getData = (x, y) => { var _a; return ((_a = data[4 * (y * w + x) + 3]) !== null && _a !== void 0 ? _a : 0) / 255; };
    // const getData = (x: number, y: number) => (data[y * w + x] ?? 0) / 255;
    for (let y = 0; y < h; y++) {
        for (let x = 0; x < w; x++) {
            const a = getData(x, y);
            const i = (y + pad) * wp + x + pad;
            if (a >= 254 / 255) {
                // Fix for bad rasterizer rounding
                data[4 * (y * w + x) + 3] = 255;
                outer[i] = 0;
                inner[i] = tiny_sdf_1.INF;
            }
            else if (a > 0) {
                const d = 0.5 - a;
                outer[i] = d > 0 ? d * d : 0;
                inner[i] = d < 0 ? d * d : 0;
            }
        }
    }
};
// Convert grayscale glyph to SDF using pixel-based distance transform
const glyphToEDT = (data, w, h, pad = 4, radius = 3, cutoff = 0.25) => {
    const wp = w + pad * 2;
    const hp = h + pad * 2;
    const np = wp * hp;
    const sp = Math.max(wp, hp);
    const out = new Uint8Array(np);
    const stage = (0, tiny_sdf_1.getSDFStage)(sp);
    paintIntoStage(stage, data, w, h, pad);
    const { outer, inner, f, z, v } = stage;
    (0, tiny_sdf_1.edt)(outer, 0, 0, wp, hp, wp, f, z, v);
    (0, tiny_sdf_1.edt)(inner, pad, pad, w, h, wp, f, z, v);
    for (let i = 0; i < np; i++) {
        const d = Math.sqrt(outer[i]) - Math.sqrt(inner[i]);
        out[i] = Math.max(0, Math.min(255, Math.round(255 - 255 * (d / radius + cutoff))));
    }
    return (0, tiny_sdf_1.glyphToRGBA)(out, wp, hp);
};
exports.glyphToEDT = glyphToEDT;
//# sourceMappingURL=sdf-edt.js.map