"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.entityToSerializedNodes = entityToSerializedNodes;
const components_1 = require("../../components");
const transform_1 = require("./transform");
function entityToSerializedNodes(entity) {
    const id = entity.__id;
    const parentId = entity.has(components_1.Children)
        ? entity.read(components_1.Children).parent.__id
        : undefined;
    let type;
    let attributes = {};
    if (entity.has(components_1.Circle)) {
        type = 'circle';
        const { cx, cy, r } = entity.read(components_1.Circle);
        Object.assign(attributes, {
            cx,
            cy,
            r,
        });
    }
    else if (entity.has(components_1.Ellipse)) {
        type = 'ellipse';
        const { cx, cy, rx, ry } = entity.read(components_1.Ellipse);
        Object.assign(attributes, {
            cx,
            cy,
            rx,
            ry,
        });
    }
    else if (entity.has(components_1.Rect)) {
        type = 'rect';
        const { x, y, width, height, cornerRadius } = entity.read(components_1.Rect);
        Object.assign(attributes, {
            x,
            y,
            width,
            height,
            cornerRadius,
        });
    }
    else if (entity.has(components_1.Polyline)) {
        type = 'polyline';
        const { points } = entity.read(components_1.Polyline);
        Object.assign(attributes, {
            points,
        });
    }
    else if (entity.has(components_1.Path)) {
        type = 'path';
        const { d, fillRule, tessellationMethod } = entity.read(components_1.Path);
        Object.assign(attributes, {
            d,
            fillRule,
            tessellationMethod,
        });
    }
    else if (entity.has(components_1.Text)) {
        type = 'text';
        const { x, y, content, fontFamily, fontSize, fontWeight, fontStyle, fontVariant, letterSpacing, lineHeight, whiteSpace, wordWrap, wordWrapWidth, textOverflow, maxLines, textAlign, textBaseline, leading, bitmapFont, bitmapFontKerning, physical, esdt, } = entity.read(components_1.Text);
        Object.assign(attributes, {
            x,
            y,
            content,
            fontFamily,
            fontSize,
            fontWeight,
            fontStyle,
            fontVariant,
            letterSpacing,
            lineHeight,
            whiteSpace,
            wordWrap,
            wordWrapWidth,
            textOverflow,
            maxLines,
            textAlign,
            textBaseline,
            leading,
            bitmapFont,
            bitmapFontKerning,
            physical,
            esdt,
        });
    }
    else {
        type = 'g';
    }
    if (entity.has(components_1.FillSolid)) {
        attributes.fill = entity.read(components_1.FillSolid).value;
    }
    else if (entity.has(components_1.FillGradient)) {
        // TODO: serialize gradient
    }
    if (entity.has(components_1.Stroke)) {
        const { color, width, alignment, linecap, linejoin, miterlimit, dasharray, dashoffset, } = entity.read(components_1.Stroke);
        Object.assign(attributes, {
            stroke: color,
            strokeWidth: width,
            strokeAlignment: alignment,
            strokeLinecap: linecap,
            strokeLinejoin: linejoin,
            strokeMiterlimit: miterlimit,
            strokeDasharray: [dasharray[0], dasharray[1]],
            strokeDashoffset: dashoffset,
        });
    }
    if (entity.has(components_1.Opacity)) {
        const { opacity, fillOpacity, strokeOpacity } = entity.read(components_1.Opacity);
        Object.assign(attributes, {
            opacity,
            fillOpacity,
            strokeOpacity,
        });
    }
    // serialize transform
    if (entity.has(components_1.Transform)) {
        attributes.transform = (0, transform_1.serializeTransform)(entity.read(components_1.Transform));
    }
    if (entity.has(components_1.Name)) {
        attributes.name = entity.read(components_1.Name).value;
    }
    // serialize visibility
    attributes.visibility = entity.has(components_1.Visibility)
        ? entity.read(components_1.Visibility).value
        : 'visible';
    // serialize children
    const nodes = [
        Object.assign({ id: `${id}`, parentId: parentId ? `${parentId}` : undefined, type }, attributes),
    ];
    if (entity.has(components_1.Parent)) {
        const children = entity.read(components_1.Parent).children.map(entityToSerializedNodes);
        nodes.push(...children.flat(1));
    }
    return nodes;
}
//# sourceMappingURL=entity.js.map