"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.serializeImage = serializeImage;
exports.serializeCanvasImageSource = serializeCanvasImageSource;
const environment_1 = require("../../environment");
const __1 = require("..");
/**
 * Serialize an image to a data URL with Canvas2D API.
 */
function serializeImage(source) {
    /**
     * We can't use bitmaprenderer since the ImageBitmap has been rendered in Texture.
     *
     * Error message: `The input ImageBitmap has been detached`
     * @see https://stackoverflow.com/questions/52959839/convert-imagebitmap-to-blob
     */
    const canvas = environment_1.DOMAdapter.get().createCanvas(source.width, source.height);
    // We get the 2d drawing context and draw the image in the top left
    canvas.getContext('2d').drawImage(source, 0, 0);
    // get a bitmaprenderer context
    // const ctx = canvas.getContext('bitmaprenderer');
    // ctx.transferFromImageBitmap(bmp);
    // const blob = await new Promise<Blob>((res) => canvas.toBlob(res));
    return canvas.toDataURL();
}
function serializeCanvasImageSource(source) {
    if (__1.isBrowser) {
        if (source instanceof ImageBitmap) {
            return serializeImage(source);
        }
        else if (source instanceof HTMLImageElement) {
            return source.src;
        }
        else if (source instanceof HTMLCanvasElement) {
            return source.toDataURL();
        }
        else if (source instanceof HTMLVideoElement) {
            // won't support
        }
    }
    else {
        return serializeImage(source);
    }
}
//# sourceMappingURL=image.js.map