"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.serializeNodesToSVGElements = serializeNodesToSVGElements;
exports.camelToKebabCase = camelToKebabCase;
exports.exportInnerShadow = exportInnerShadow;
exports.exportDropShadow = exportDropShadow;
exports.exportText = exportText;
const util_1 = require("@antv/util");
const toposort_1 = __importDefault(require("toposort"));
const __1 = require("../..");
const browser_1 = require("../browser");
const points_1 = require("./points");
/**
 * No need to output default value in SVG Element.
 */
const defaultValues = {
    opacity: 1,
    fillOpacity: 1,
    strokeOpacity: 1,
    fill: 'black',
    stroke: 'none',
    strokeWidth: 1,
    strokeLinecap: 'butt',
    strokeLinejoin: 'miter',
    strokeAlignment: 'center',
    strokeMiterlimit: 4,
    strokeDasharray: 'none',
    strokeDashoffset: 0,
    innerShadowBlurRadius: 0,
    innerShadowColor: 'none',
    innerShadowOffsetX: 0,
    innerShadowOffsetY: 0,
    visibility: 'visible',
    transform: 'matrix(1,0,0,1,0,0)',
    cornerRadius: 0,
    dropShadowColor: 'none',
    dropShadowOffsetX: 0,
    dropShadowOffsetY: 0,
    dropShadowBlurRadius: 0,
    fillRule: 'nonzero',
};
// @see https://github.com/plouc/nivo/issues/164
const BASELINE_MAP = {
    top: 'hanging', // Use hanging here.
    middle: 'central',
    bottom: 'text-after-edge', // FIXME: It is not a standard property.
    alphabetic: 'alphabetic',
    ideographic: 'ideographic',
    hanging: 'hanging',
};
function serializeNodesToSVGElements(nodes) {
    const elements = [];
    const idSerializedNodeMap = new Map();
    for (const node of nodes) {
        idSerializedNodeMap.set(node.id, node);
    }
    const idSVGElementMap = new Map();
    const vertices = nodes.map((node) => node.id);
    const edges = nodes
        .filter((node) => !(0, util_1.isNil)(node.parentId))
        .map((node) => [node.parentId, node.id]);
    const sorted = toposort_1.default.array(vertices, edges);
    for (const id of sorted) {
        const node = idSerializedNodeMap.get(id);
        const { id: _, parentId, type } = node, restAttributes = __rest(node, ["id", "parentId", "type"]);
        const element = (0, browser_1.createSVGElement)(type);
        element.id = `${id}`;
        const _a = restAttributes, { transform, innerShadowColor, innerShadowOffsetX, innerShadowOffsetY, innerShadowBlurRadius, dropShadowColor, dropShadowOffsetX, dropShadowOffsetY, dropShadowBlurRadius, strokeAlignment, tessellationMethod, cornerRadius, zIndex, fontFamily, fontSize, fontWeight, fontStyle, fontVariant, content, letterSpacing, lineHeight, whiteSpace, wordWrap, wordWrapWidth, textAlign, textBaseline, bitmapFontKerning, physical, esdt, leading, maxLines } = _a, rest = __rest(_a, ["transform", "innerShadowColor", "innerShadowOffsetX", "innerShadowOffsetY", "innerShadowBlurRadius", "dropShadowColor", "dropShadowOffsetX", "dropShadowOffsetY", "dropShadowBlurRadius", "strokeAlignment", "tessellationMethod", "cornerRadius", "zIndex", "fontFamily", "fontSize", "fontWeight", "fontStyle", "fontVariant", "content", "letterSpacing", "lineHeight", "whiteSpace", "wordWrap", "wordWrapWidth", "textAlign", "textBaseline", "bitmapFontKerning", "physical", "esdt", "leading", "maxLines"]);
        Object.entries(rest).forEach(([key, value]) => {
            if (`${value}` !== '' && `${defaultValues[key]}` !== `${value}`) {
                element.setAttribute(camelToKebabCase(key), `${value}`);
            }
        });
        // Handle negative size of rect.
        if (type === 'rect') {
            const { width, height, x, y } = node;
            if (width < 0 || height < 0) {
                const x1 = x;
                const y1 = y;
                const x2 = x + width;
                const y2 = y + height;
                element.setAttribute('x', `${Math.min(x1, x2)}`);
                element.setAttribute('y', `${Math.min(y1, y2)}`);
                element.setAttribute('width', `${Math.abs(width)}`);
                element.setAttribute('height', `${Math.abs(height)}`);
            }
        }
        if (textAlign) {
            // "center" | "end" | "left" | "right" | "start";
            // @see https://developer.mozilla.org/en-US/docs/Web/SVG/Reference/Attribute/text-anchor
            if (textAlign === 'center') {
                element.setAttribute('text-anchor', 'middle');
            }
            else {
                element.setAttribute('text-anchor', textAlign);
            }
        }
        if (textBaseline) {
            element.setAttribute('dominant-baseline', BASELINE_MAP[textBaseline]);
        }
        const innerStrokeAlignment = strokeAlignment === 'inner';
        const outerStrokeAlignment = strokeAlignment === 'outer';
        const innerOrOuterStrokeAlignment = innerStrokeAlignment || outerStrokeAlignment;
        // const hasFillImage = rest.fill && isString(rest.fill) && isDataUrl(rest.fill);
        // const hasFillGradient =
        //   rest.fill && isString(rest.fill) && isGradient(rest.fill);
        // const hasFillPattern = rest.fill && isPattern(rest.fill);
        const hasFillImage = false;
        const hasFillGradient = false;
        const hasFillPattern = false;
        const isRough = false;
        const visible = true;
        const hasChildren = edges.some(([parentId]) => parentId === id);
        /**
         * In the vast majority of cases, it is the element itself.
         *
         * Here's 3 examples where it's not the element itself but a <g> element as its parent.
         * @example
         *
         * When the element has children.
         * ```html
         * <g transform="matrix()">
         *  <circle />
         *  <text /> <!-- child #1 -->
         *  <text /> <!-- child #2 -->
         * </g>
         * ```
         *
         * When strokeAlignment is 'inner' or 'outer'.
         * ```html
         * <g transform="matrix()">
         *  <circle /> <!-- fill -->
         *  <circle /> <!-- stroke -->
         * </g>
         * ```
         *
         * `innerShadow` is implemented as a filter effect.
         * ```html
         * <g filter="url(#filter)">
         *   <defs>
         *     <filter id="filter" />
         *   </defs>
         *   <circle />
         * </g>
         * ```
         */
        let $g;
        if ((hasChildren && type !== 'g') ||
            (innerOrOuterStrokeAlignment && type !== 'polyline') ||
            isRough ||
            hasFillImage ||
            hasFillGradient ||
            hasFillPattern) {
            $g = (0, browser_1.createSVGElement)('g');
            if (element) {
                $g.appendChild(element);
            }
        }
        if (innerOrOuterStrokeAlignment) {
            exportInnerOrOuterStrokeAlignment(node, element, $g);
        }
        if (innerShadowBlurRadius > 0) {
            exportInnerShadow(node, element, $g);
        }
        if (dropShadowBlurRadius > 0) {
            // RoughRect has no element, use $g instead.
            exportDropShadow(node, element || $g, $g);
        }
        // avoid `fill="[object ImageBitmap]"`
        if (hasFillImage) {
            // exportFillImage(node, element, $g);
        }
        if (hasFillGradient || hasFillPattern) {
            // exportFillGradientOrPattern(node, element, $g);
        }
        $g = $g || element;
        // if (visible === false) {
        //   // @see https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/visibility
        //   $g.setAttribute('visibility', 'hidden');
        // }
        if (cornerRadius) {
            $g.setAttribute('rx', `${cornerRadius}`);
            $g.setAttribute('ry', `${cornerRadius}`);
        }
        if (isRough) {
            // exportRough(node, $g);
        }
        if (content) {
            exportText(node, $g);
        }
        const { a, b, c, d, tx, ty } = transform.matrix;
        if (a !== 1 || b !== 0 || c !== 0 || d !== 1 || tx !== 0 || ty !== 0) {
            $g.setAttribute('transform', `matrix(${a},${b},${c},${d},${tx},${ty})`);
        }
        // TODO: ZIndex
        idSVGElementMap.set(id, $g);
        if (parentId) {
            const parent = idSVGElementMap.get(parentId);
            if (parent) {
                parent.appendChild($g);
            }
        }
        else {
            elements.push($g);
        }
    }
    return elements;
}
function camelToKebabCase(str) {
    return str.replace(/[A-Z]/g, (match) => `-${match.toLowerCase()}`);
}
/**
 * @see https://stackoverflow.com/questions/74958705/how-to-simulate-stroke-align-stroke-alignment-in-svg
 * @example
 * ```html
 * <g transform="matrix()">
 *  <circle /> <!-- fill -->
 *  <circle /> <!-- stroke -->
 * </g>
 * ```
 */
function exportInnerOrOuterStrokeAlignment(attributes, element, $g) {
    const { type } = attributes;
    const { strokeWidth, strokeAlignment } = attributes;
    const innerStrokeAlignment = strokeAlignment === 'inner';
    const halfStrokeWidth = strokeWidth / 2;
    if (type === 'polyline') {
        const { points: pointsStr } = attributes;
        const points = pointsStr
            .split(' ')
            .map((xy) => xy.split(',').map(Number));
        const shiftedPoints = (0, __1.shiftPoints)(points, innerStrokeAlignment, strokeWidth);
        element.setAttribute('points', (0, points_1.serializePoints)(shiftedPoints));
    }
    else {
        const $stroke = element.cloneNode();
        element.setAttribute('stroke', 'none');
        $stroke.setAttribute('fill', 'none');
        if (type === 'circle') {
            const { r } = attributes;
            const offset = innerStrokeAlignment ? -halfStrokeWidth : halfStrokeWidth;
            $stroke.setAttribute('r', `${r + offset}`);
        }
        else if (type === 'ellipse') {
            const { rx, ry } = attributes;
            const offset = innerStrokeAlignment ? -halfStrokeWidth : halfStrokeWidth;
            $stroke.setAttribute('rx', `${rx + offset}`);
            $stroke.setAttribute('ry', `${ry + offset}`);
        }
        else if (type === 'rect') {
            const { x, y, width, height, strokeWidth } = attributes;
            $stroke.setAttribute('x', `${x + (innerStrokeAlignment ? halfStrokeWidth : -halfStrokeWidth)}`);
            $stroke.setAttribute('y', `${y + (innerStrokeAlignment ? halfStrokeWidth : -halfStrokeWidth)}`);
            $stroke.setAttribute('width', `${width + (innerStrokeAlignment ? -strokeWidth : strokeWidth)}`);
            $stroke.setAttribute('height', `${height + (innerStrokeAlignment ? -strokeWidth : strokeWidth)}`);
        }
        $g.appendChild($stroke);
    }
}
/**
 * Use filter to create inner shadow.
 * @see https://stackoverflow.com/questions/69799051/creating-inner-shadow-in-svg
 * @example
 * ```html
 * <g filter="url(#filter)">
 *   <defs>
 *     <filter id="filter">
 *       <feOffset dx="10" dy="10"/>
 *     </filter>
 *   </defs>
 *   <circle />
 * </g>
 * ```
 */
function exportInnerShadow(attributes, element, $g) {
    const { id } = attributes;
    const { innerShadowOffsetX, innerShadowOffsetY, innerShadowBlurRadius, innerShadowColor, } = attributes;
    const $defs = (0, browser_1.createSVGElement)('defs');
    const $filter = (0, browser_1.createSVGElement)('filter');
    $filter.id = `filter_inner_shadow_${id}`;
    const $feComponentTransfer = (0, browser_1.createSVGElement)('feComponentTransfer');
    $feComponentTransfer.setAttribute('in', 'SourceAlpha');
    const $feFuncA = (0, browser_1.createSVGElement)('feFuncA');
    $feFuncA.setAttribute('type', 'table');
    $feFuncA.setAttribute('tableValues', '1 0');
    $feComponentTransfer.appendChild($feFuncA);
    $filter.appendChild($feComponentTransfer);
    const $feGaussianBlur = (0, browser_1.createSVGElement)('feGaussianBlur');
    $feGaussianBlur.setAttribute('stdDeviation', `${(innerShadowBlurRadius || 0) / 4}`);
    $filter.appendChild($feGaussianBlur);
    const $feOffset = (0, browser_1.createSVGElement)('feOffset');
    $feOffset.setAttribute('dx', `${(innerShadowOffsetX || 0) / 2}`);
    $feOffset.setAttribute('dy', `${(innerShadowOffsetY || 0) / 2}`);
    $feOffset.setAttribute('result', 'offsetblur');
    $filter.appendChild($feOffset);
    const $feFlood = (0, browser_1.createSVGElement)('feFlood');
    $feFlood.setAttribute('flood-color', innerShadowColor);
    $feFlood.setAttribute('result', 'color');
    $filter.appendChild($feFlood);
    const $feComposite = (0, browser_1.createSVGElement)('feComposite');
    $feComposite.setAttribute('in2', 'offsetblur');
    $feComposite.setAttribute('operator', 'in');
    $filter.appendChild($feComposite);
    const $feComposite2 = (0, browser_1.createSVGElement)('feComposite');
    $feComposite2.setAttribute('in2', 'SourceAlpha');
    $feComposite2.setAttribute('operator', 'in');
    $filter.appendChild($feComposite2);
    const $feMerge = (0, browser_1.createSVGElement)('feMerge');
    $filter.appendChild($feMerge);
    const $feMergeNode = (0, browser_1.createSVGElement)('feMergeNode');
    $feMergeNode.setAttribute('in', 'SourceGraphic');
    const $feMergeNode2 = (0, browser_1.createSVGElement)('feMergeNode');
    $feMerge.appendChild($feMergeNode);
    $feMerge.appendChild($feMergeNode2);
    $defs.appendChild($filter);
    element.appendChild($defs);
    const existedFilter = element.getAttribute('filter') || '';
    element.setAttribute('filter', `${existedFilter} url(#${$filter.id})`.trim());
}
function exportDropShadow(attributes, element, $g) {
    const { id, width, height, dropShadowBlurRadius, dropShadowColor, dropShadowOffsetX, dropShadowOffsetY, } = attributes;
    const $defs = (0, browser_1.createSVGElement)('defs');
    const $filter = (0, browser_1.createSVGElement)('filter');
    $filter.id = `filter_drop_shadow_${id}`;
    const $feDropShadow = (0, browser_1.createSVGElement)('feDropShadow');
    $feDropShadow.setAttribute('dx', `${((dropShadowOffsetX || 0) / 2) * Math.sign(width)}`);
    $feDropShadow.setAttribute('dy', `${((dropShadowOffsetY || 0) / 2) * Math.sign(height)}`);
    $feDropShadow.setAttribute('stdDeviation', `${(dropShadowBlurRadius || 0) / 4}`);
    $feDropShadow.setAttribute('flood-color', dropShadowColor);
    $filter.appendChild($feDropShadow);
    $defs.appendChild($filter);
    element.appendChild($defs);
    const existedFilter = element.getAttribute('filter') || '';
    element.setAttribute('filter', `${existedFilter} url(#${$filter.id})`.trim());
}
/**
 * use <text> and <tspan> to render text.
 * @see https://developer.mozilla.org/en-US/docs/Web/SVG/Element/text#example
 */
function exportText(attributes, $g) {
    const { content, fontFamily, fontSize, fontWeight, fontStyle, fontVariant, fill, } = attributes;
    $g.textContent = content;
    let styleCSSText = '';
    const fontStyleString = (0, __1.fontStringFromTextStyle)({
        fontFamily,
        fontSize,
        fontWeight,
        fontStyle,
        fontVariant,
    });
    if (fontStyleString) {
        styleCSSText += `font: ${fontStyleString};`;
    }
    if (fill) {
        styleCSSText += `fill: ${fill};`;
    }
    if (styleCSSText) {
        $g.setAttribute('style', styleCSSText);
    }
}
//# sourceMappingURL=svg.js.map